// ClassTreeDataModel.java

package org.javalobby.cafebabe.plugin.classplug;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.tree.*;

import treetable.*;
import org.javalobby.classfile.*;
import org.javalobby.classfile.attribute.*;

import org.javalobby.cafebabe.*;

// Data model for ClassTreeTable object

public class ClassTreeDataModel extends DefaultTreeModel
                                implements TreeTableModel, ClassPlugConstants {

  private final int FOLDERS_NUMBER = FOLDERS_NAME.length;

  private FolderNode[] folders = new FolderNode[FOLDERS_NUMBER];

  private RootNode root;

  private ClassFile classFile;
  private ConstPool constPool;

  public ClassTreeDataModel(boolean[] foldersVisible) {
    super(new RootNode());

    root = (RootNode)getRoot();

    // Create folders
    for(int i=0; i < FOLDERS_NUMBER; i++)
      folders[i] = new FolderNode(FOLDERS_NAME[i], foldersVisible[i]);

    createTree();
  }

  public void setClassFile(ClassFile classFile) {
    this.classFile = classFile;

    if(classFile != null)
      this.constPool = classFile.getConstPool();
    else
      this.constPool = null;
  }

  public void updateClassAttributes() {
    updateFolder(folders[10], classFile.attributes());
    updateFolder(folders[3], constPool);
  }

  public void updateMethods() {
    updateFolder(folders[9], classFile.methods());
  }

  // if subtree is marked as "hidden" we delete it,
  // otherwise -- create and update it
  public void showFolders(boolean[] foldersVisible) {
    if(classFile == null)
      checkEmptyFolders(foldersVisible);
    else
      checkFullFolders(foldersVisible);

    sortFolders();
  }

  private void checkEmptyFolders(boolean[] foldersVisible) {
    for(int i=0; i < FOLDERS_NUMBER; i++) {
      if(foldersVisible[i] != folders[i].isVisible()) {

        folders[i].setVisible(foldersVisible[i]);

        if(foldersVisible[i])
          createSubtree(folders[i]);
        else {
          deleteSubtree(folders[i]);
          this.removeNodeFromParent(folders[i]);
        }
      }
    }
  }

  private void checkFullFolders(boolean[] foldersVisible) {
    checkSimpleSubtree(folders[0], foldersVisible[0], getMagicNumber(), getMagicNumber());
    checkSimpleSubtree(folders[1], foldersVisible[1], getMagicNumber(), getMagicNumber());
    checkSimpleSubtree(folders[2], foldersVisible[2], getMajorVersion(),getMajorVersion());
    checkComplexSubtree(folders[3], foldersVisible[3], constPool);
    checkSimpleSubtree(folders[4], foldersVisible[4],classFile.getAccessFlags().toString(),
                       getAccessFlags());
    checkSimpleSubtree(folders[5], foldersVisible[5],    getThisName(), getThisClass());
    checkSimpleSubtree(folders[6], foldersVisible[6],    getSuperName(), getSuperClass());
    checkComplexSubtree(folders[7], foldersVisible[7],   classFile.interfaces());
    checkComplexSubtree(folders[8], foldersVisible[8],   classFile.fields());
    checkComplexSubtree(folders[9], foldersVisible[9],   classFile.methods());
    checkComplexSubtree(folders[10], foldersVisible[10], classFile.attributes());
  }

  private void checkSimpleSubtree(FolderNode folder, boolean b,
                                  String text1, String text2) {
    if(b != folder.isVisible()) {
      folder.setVisible(b);

      if(b) {
        createSubtree(folder);
        updateSubtree(folder, text1, text2);
      }
      else {
        deleteSubtree(folder);
        this.removeNodeFromParent(folder);
        folder.setInfo("");
      }
    }
  }

  private void checkComplexSubtree(FolderNode folder, boolean b, EntryCollection entries) {
    if(b != folder.isVisible()) {
      folder.setVisible(b);

      if(b) {
        createSubtree(folder);
        modifySubtree(folder, entries);
      }
      else {
        deleteSubtree(folder);
        this.removeNodeFromParent(folder);
        folder.setInfo("");
      }
    }
  }

  public void updateNode(LeafNode leaf, PoolEntry entry, String newValue) {
    String line = leaf.getUserObject().toString();
    int pos = line.indexOf(" ");
    String indexString = line.substring(0, pos);
    leaf.setText(indexString + " " + entry.getType() + " " + newValue);
    leaf.setInfo(Converter.toHexString(BytesProducer.getBytes(entry)));
  }

  private void updateSubtree(FolderNode folder, String text1, String text2) {
    if(folder.isVisible()) {
      LeafNode leafNode = (LeafNode)folder.getChildAt(0);
      leafNode.setText(text1);
      folder.setInfo(text2);
    }
  }

  private void updateFolder(FolderNode folder, EntryCollection entries) {
    if(folder.isVisible()) {
      deleteSubtree(folder);
      this.removeNodeFromParent(folder);
      folder.setInfo("");

      createSubtree(folder);
      if(classFile != null)
        modifySubtree(folder, entries);
    }
    sortFolders();
  }

  // we create each "visible" subtree
  public void createTree() {
   for(int i=0; i < FOLDERS_NUMBER; i++)
     if(folders[i].isVisible())
       createSubtree(folders[i]);
  }

  public void deleteTree() {
    for(int i=0; i < FOLDERS_NUMBER; i++) {
      if(folders[i].isVisible()) {
        for(int cnt=folders[i].getChildCount(), j=0; j < cnt; j++) {
          MutableTreeNode n = (MutableTreeNode)(folders[i].getChildAt(0));
          this.removeNodeFromParent(n);
        }

        this.removeNodeFromParent(folders[i]);
        folders[i].setInfo("");
      }
    }
  }

  // we update each "visible" subtree
  public void updateTree() {
    updateSubtree(folders[0],  getMagicNumber(), getMagicNumber());
    updateSubtree(folders[1],  getMinorVersion(), getMinorVersion());
    updateSubtree(folders[2],  getMajorVersion(), getMajorVersion());
    modifySubtree(folders[3],  constPool);
    updateSubtree(folders[4],  classFile.getAccessFlags().toString(), getAccessFlags());
    updateSubtree(folders[5],  getThisName(), getThisClass());
    updateSubtree(folders[6],  getSuperName(), getSuperClass());
    modifySubtree(folders[7],  classFile.interfaces());
    modifySubtree(folders[8],  classFile.fields());
    modifySubtree(folders[9],  classFile.methods());
    modifySubtree(folders[10], classFile.attributes());
  }

  private void sortFolders() {
    int cnt = root.getChildCount();
    MutableTreeNode[] fs = new MutableTreeNode[cnt];

    for(int i=0; i < cnt; i++) {
      MutableTreeNode folder = (MutableTreeNode)root.getChildAt(0);
      fs[i] = folder;
      this.removeNodeFromParent(folder);
    }

    for(int i=0; i < FOLDERS_NUMBER; i++)
      if(folders[i].isVisible())
        this.insertNodeInto(folders[i], root, root.getChildCount());
  }


  private void createSubtree(FolderNode folder) {
    LeafNode leaf = new LeafNode("unresolved");
    this.insertNodeInto(leaf, folder, 0);
    this.insertNodeInto(folder, root, root.getChildCount());
  }

  private void deleteSubtree(MutableTreeNode node) {
    for(int cnt=node.getChildCount(), i=0; i < cnt; i++) {
      TreeTableNode n = (TreeTableNode)(node.getChildAt(0));
      if(!n.isLeaf())
        deleteSubtree(n);

      this.removeNodeFromParent(n);
      n.setInfo("");
    }
  }

  private void modifySubtree(FolderNode folder, EntryCollection entries) {
    if(folder.isVisible()) {
      deleteSubtree(folder);

      if(entries.size() == 0) {
        this.insertNodeInto(new LeafNode("none"), folder, 0);
      }
      else {
        for(short i=0; i < entries.size(); i++) {
          Entry entry = (Entry)entries.elementAt(i);
          insert(folder, entry, i);
        }
      }
      folder.setInfo(getHexSize(entries));
    }
  }

  private void insert(FolderNode folder, Entry entry, short index) {
    if(entry instanceof PoolEntry) {
      insert(folder, (PoolEntry)entry, index);
    }
    else if(entry instanceof InterfaceEntry) {
      insert(folder, (InterfaceEntry)entry);
    }
    else if(entry instanceof FieldEntry) {
      insert(folder, new FieldNode((FieldEntry)entry), (MemberEntry)entry);
    }
    else if(entry instanceof MethodEntry) {
      insert(folder, new MethodNode((MethodEntry)entry), (MemberEntry)entry);
    }
    else if(entry instanceof AttributeEntry) {
      insert(folder, (AttributeEntry)entry);
    }
  }

  private void insert(FolderNode folder, PoolEntry constant, short index) {
    if(constant == null) return;

    String[] str = constant.resolve(constPool);

    int cnt = folder.getChildCount();
    LeafNode leaf = new LeafNode(Converter.toHexString((short)index, "") + "  " +
                                   Converter.merge(str, " "));
    this.insertNodeInto(leaf, folder, cnt);
    leaf.setInfo(Converter.toHexString(BytesProducer.getBytes(constant)));
  }

  private void insert(FolderNode folder, InterfaceEntry entry) {
    String[] str = entry.resolve(constPool);
    LeafNode leaf = new LeafNode(str[0]);
    this.insertNodeInto(leaf, folder, folder.getChildCount());
    leaf.setInfo(Converter.toHexString(entry.getNameIndex()));
  }

  private void insert(FolderNode folder, TreeTableNode node, MemberEntry entry) {
    String[] str = entry.resolve(constPool);

    this.insertNodeInto(node, folder, folder.getChildCount());

    node.setText(Converter.merge(str, " "));
    node.setInfo(
      Converter.toHexString(entry.getAccessFlags().getAccessFlags()) + " " +
      Converter.toHexString(entry.getNameIndex()) + " " +
      Converter.toHexString(entry.getValueIndex()));

    EntryCollection attributes1 = entry.getAttributes();
    short size = (short)attributes1.size();

    FolderNode attrFolder = new FolderNode("Attributes");
    this.insertNodeInto(attrFolder, node, 0);
    attrFolder.setInfo(Converter.toHexString(size));

    for(short i=0; i < size; i++) {
      AttributeEntry entry1 = (AttributeEntry)attributes1.elementAt(i);
      insert(attrFolder, entry1);
    }
  }

  private void insert(FolderNode folder, AttributeEntry attribute) {
    String attrName = attribute.getName(constPool);

    if(attrName.equals(AttributeEntry.SOURCE_FILE)) {
      insert(folder, new SourceFileAttribute(attribute));
    }
    else if(attrName.equals(AttributeEntry.CONSTANT_VALUE)) {
      insert(folder, new ConstantValueAttribute(attribute));
    }
    else if(attrName.equals(AttributeEntry.INNER_CLASSES)) {
      insert(folder, new InnerClassesAttribute(attribute));
    }
    else if(attrName.equals(AttributeEntry.EXCEPTIONS)) {
      insert(folder, new ExceptionsAttribute(attribute));
    }
    else if(attrName.equals(AttributeEntry.CODE)) {
      insert(folder, new CodeAttribute(attribute));
    }
    else if(attrName.equals(AttributeEntry.LINE_NUMBER_TABLE)) {
      insert(folder, new LineNumberTableAttribute(attribute));
    }
    else if(attrName.equals(AttributeEntry.LOCAL_VARIABLE_TABLE)) {
      insert(folder, new LocalVariableTableAttribute(attribute));
    }
    else {
      insertUnknown(folder, attribute);
    }
  }

  private void insertUnknown(FolderNode folder, AttributeEntry attribute) {
    byte[] buffer = attribute.getBuffer();
    String name   = attribute.getName(constPool);

    FolderNode attrNode = new FolderNode(name);
    this.insertNodeInto(attrNode, folder, folder.getChildCount());
    attrNode.setInfo(
      Converter.toHexString(attribute.getNameIndex()) + " " +
      Converter.toHexString(buffer.length));

    String text = null;

    if(buffer.length == 2) {
      short index = (short)((buffer[0] & 0xff) << 8 | (buffer[1] & 0xff));
      if(index < constPool.size()) {
        PoolEntry pe = constPool.getEntry(index);

        if(pe instanceof Utf8Const) {
          Utf8Const utf8Const = (Utf8Const)pe;

          text = utf8Const.getValue();
        }
        else {
          text = Converter.toHexString(buffer);
        }
      }
      else {
        text = Converter.toHexString(buffer);
      }
    }
    else if(buffer.length > 0) {
      text = Converter.toHexString(buffer);
    }

    if(text != null) {
      LeafNode leaf = new LeafNode(text);
      this.insertNodeInto(leaf, attrNode, attrNode.getChildCount());
      leaf.setInfo(text);
    }
  }

  private void insert(FolderNode folder, SourceFileAttribute attribute) {
    FolderNode attrNode = new FolderNode(AttributeEntry.SOURCE_FILE);
    this.insertNodeInto(attrNode, folder, folder.getChildCount());
    attrNode.setInfo(
      Converter.toHexString(attribute.getNameIndex()) + " " +
      Converter.toHexString(attribute.getBuffer().length));

    short index = attribute.getIndex();

    PoolEntry entry = constPool.getEntry(index);
    String sourceFileName = entry.resolve(constPool)[1];

    LeafNode leaf = new LeafNode(sourceFileName);
    this.insertNodeInto(leaf, attrNode, attrNode.getChildCount());
    leaf.setInfo(Converter.toHexString(index));
  }

  private void insert(FolderNode folder, ConstantValueAttribute attribute) {
    FolderNode attrNode = new FolderNode(AttributeEntry.CONSTANT_VALUE);
    this.insertNodeInto(attrNode, folder, folder.getChildCount());
    attrNode.setInfo(
      Converter.toHexString(attribute.getNameIndex()) + " " +
      Converter.toHexString((short)attribute.getBuffer().length));

    short index = attribute.getIndex();

    PoolEntry entry = constPool.getEntry(index);
    String constantValue = entry.resolve(constPool)[1];

    LeafNode leaf = new LeafNode(constantValue);
    this.insertNodeInto(leaf, attrNode, attrNode.getChildCount());
    leaf.setInfo(Converter.toHexString(index));
  }

  private void insert(FolderNode folder, InnerClassesAttribute attribute) {
    InnerClassEntry[] innerClasses = attribute.getInnerClasses();

    FolderNode attrNode = new FolderNode(AttributeEntry.INNER_CLASSES);
    this.insertNodeInto(attrNode, folder, folder.getChildCount());
    attrNode.setInfo(
      Converter.toHexString(attribute.getNameIndex()) + " " +
      Converter.toHexString(attribute.getBuffer().length) + " " +
      Converter.toHexString((short)innerClasses.length)
      );

    for(int i=0; i < innerClasses.length; i++) {
      InnerClassEntry innerClass = innerClasses[i];

      String[] r = innerClass.resolve(constPool);

      LeafNode leaf = new LeafNode(r[0]);
      this.insertNodeInto(leaf, attrNode, attrNode.getChildCount());
      leaf.setInfo(Converter.toHexString(BytesProducer.getBytes(innerClass)));
    }
  }

  private void insert(FolderNode folder, ExceptionsAttribute attribute) {
    ExceptionEntry[] exceptions = attribute.getExceptions();

    FolderNode attrNode = new FolderNode(AttributeEntry.EXCEPTIONS);
    this.insertNodeInto(attrNode, folder, folder.getChildCount());
    attrNode.setInfo(
      Converter.toHexString(attribute.getNameIndex()) + " " +
      Converter.toHexString(attribute.getBuffer().length) + " " +
      Converter.toHexString((short)exceptions.length));

    for(int i=0; i < exceptions.length; i++) {
      ExceptionEntry exception = exceptions[i];

      String[] r = exception.resolve(constPool);

      LeafNode leaf = new LeafNode(r[0]);
      this.insertNodeInto(leaf, attrNode, attrNode.getChildCount());
      leaf.setInfo(Converter.toHexString(exception.getNameIndex()));
    }
  }

  private void insert(FolderNode folder, CodeAttribute attribute) {
    FolderNode codeAttrNode = new FolderNode(AttributeEntry.CODE);
    this.insertNodeInto(codeAttrNode, folder, folder.getChildCount());
    codeAttrNode.setInfo(
      Converter.toHexString(attribute.getNameIndex()) + " " +
      Converter.toHexString(attribute.getBuffer().length));

    short maxStack  = attribute.getMaxStack();

    LeafNode maxStackNode = new LeafNode("maxStack");
    this.insertNodeInto(maxStackNode, codeAttrNode, codeAttrNode.getChildCount());
    maxStackNode.setInfo(Converter.toHexString(maxStack));

    short maxLocals = attribute.getMaxLocals();

    LeafNode maxLocalsNode = new LeafNode("maxLocals");
    this.insertNodeInto(maxLocalsNode, codeAttrNode, codeAttrNode.getChildCount());
    maxLocalsNode.setInfo(Converter.toHexString(maxLocals));

    MethodBody body = attribute.getMethodBody();

    LeafNode bodyNode = new LeafNode("Body");
    this.insertNodeInto(bodyNode, codeAttrNode, codeAttrNode.getChildCount());
    bodyNode.setInfo(Converter.toHexString(body.length()));

    ExceptionHandlerEntry[] exceptionHandlers = attribute.getExceptionHandlers();

    FolderNode exceptionTableNode = new FolderNode("ExceptionTable");
    this.insertNodeInto(exceptionTableNode, codeAttrNode, codeAttrNode.getChildCount());
    exceptionTableNode.setInfo(Converter.toHexString((short)exceptionHandlers.length));

    for(int i=0; i < exceptionHandlers.length; i++) {
      ExceptionHandlerEntry exceptionHandler = exceptionHandlers[i];

      String[] r = exceptionHandler.resolve(constPool);

      LeafNode leaf = new LeafNode(r[0]);
      this.insertNodeInto(leaf, exceptionTableNode, exceptionTableNode.getChildCount());
      leaf.setInfo(Converter.toHexString(BytesProducer.getBytes(exceptionHandler)));
    }

    AttributeEntry[] attributes = attribute.getAttributes();

    FolderNode attributesNode = new FolderNode("Attributes");
    this.insertNodeInto(attributesNode, codeAttrNode, codeAttrNode.getChildCount());
    attributesNode.setInfo(Converter.toHexString((short)attributes.length));

    for(int i=0; i < attributes.length; i++) {
      insert(attributesNode, attributes[i]);
    }
  }

  private void insert(FolderNode folder, LineNumberTableAttribute attribute) {
    LineNumberEntry[] lineNumbers = attribute.getLineNumbers();

    TreeTableNode attrNode = new FolderNode(AttributeEntry.LINE_NUMBER_TABLE);
    this.insertNodeInto(attrNode, folder, folder.getChildCount());
    attrNode.setInfo(
      Converter.toHexString(attribute.getNameIndex()) + " " +
      Converter.toHexString(attribute.getBuffer().length) + " " +
      Converter.toHexString((short)lineNumbers.length));

    for(int i=0; i < lineNumbers.length; i++) {
      LineNumberEntry lineNumber = lineNumbers[i];

      String title = null;
      try {
        title = lineNumber.resolve(constPool)[0];
      }
      catch(Exception e) {
        title = BAD_ENTRY_TEXT;
      }

      LeafNode leaf = new LeafNode(title);
      this.insertNodeInto(leaf, attrNode, attrNode.getChildCount());
      leaf.setInfo(Converter.toHexString(BytesProducer.getBytes(lineNumber)));
    }
  }

  private void insert(FolderNode folder, LocalVariableTableAttribute attribute) {
    LocalVariableEntry[] localVariables = attribute.getLocalVariables();

    TreeTableNode attrNode = new FolderNode(AttributeEntry.LOCAL_VARIABLE_TABLE);
    this.insertNodeInto(attrNode, folder, folder.getChildCount());
    attrNode.setInfo(
      Converter.toHexString(attribute.getNameIndex()) + " " +
      Converter.toHexString(attribute.getBuffer().length) + " " +
      Converter.toHexString((short)localVariables.length));

    for(int i=0; i < localVariables.length; i++) {
      LocalVariableEntry localVariable = localVariables[i];

      String title = null;
      try {
        title = localVariable.resolve(constPool)[0];
      }
      catch(Exception e) {
        title = BAD_ENTRY_TEXT;
      }

      LeafNode leaf = new LeafNode(title);
      this.insertNodeInto(leaf, attrNode, attrNode.getChildCount());
      leaf.setInfo(Converter.toHexString(BytesProducer.getBytes(localVariable)));
    }
  }

  private String magicNumberTip  = "Magic number (4 bytes)";
  private String minorVersionTip = "Minor version (short)";
  private String majorVersionTip = "Major version (short)";

  private String thisClassTip    = "Name of this class (short)";
  private String superClassTip   = "Name of super class (short)";

  private String constPoolTip    = "Size of constant pool (short)";
  private String interfacesTip   = "Number of interfaces (short)";
  private String fieldsTip       = "Number of fields (short)";
  private String methodsTip      = "Number of methods (short)";
  private String exceptionTableTip = "Number of exception handlers (short)";
  private String bodyTip         = "Size of code buffer (int)";

  private String classAttrTip    = "Number of class attributes (short)";
  private String fieldAttrTip    = "Number of field attributes (short)";
  private String methodAttrTip   = "Number of method attributes (short)";
  private String codeAttrTip     = "Number of code attributes (short)";
  private String sourceFileAttrTip = "Name and buffer's size (short, int)";
  private String innerClassesAttrTip = "Name, buffer's size, number of inner " +
                                   "classes (short, int, short)";

  private String innerClassTip   = "qualified name, embraced class name, " +
                                   "short name and access flags " +
                                   "(short, short, short, short)";
  private String sourceFileTip   = "Name of source file (short)";
  private String fieldTip        = "Access flags, name, signature " +
                                   "(short, short, short)";
  private String methodTip       = fieldTip;
  private String codeTip         = "Name, buffer's size (short, int)";
  private String exceptionsTip   = "Name, buffer's size, number of exceptions " +
                                   "(short, int, short)";
  private String lineNumbAttrTip = "Name, buffer's size, number of lines " +
                                   "(short, int, short)";
  private String localVarAttrTip = "Name, buffer's size, number of variables " +
                                   "(short, int, short)";
  private String syntheticAttrTip     = codeTip;
  private String constantValueAttrTip = codeTip;

  private String exceptionHandlerTip = "startPc, endPc, handlerPc, catchType " +
                                       "(short, short, short, short)";
  private String lineNumbTip = "startPc, lineNumber (short, short)";
  private String localVarTip = "startPc, length, nameIndex, descrIndex, " +
                               "index (short, short, short, short, short,)";
  private String exceptionTip = "Name of exception (short)";

  public String getToolTipText(TreeNode node) {
    if(classFile != null) {
      String nodeText = node.toString();

      if(nodeText.equals(MAGIC_NUMBER_TEXT)) {
        return magicNumberTip;
      }
      else if(nodeText.equals(MINOR_VERSION_TEXT)) {
        return minorVersionTip;
      }
      else if(nodeText.equals(MAJOR_VERSION_TEXT)) {
        return majorVersionTip;
      }
      else if(nodeText.equals(CONSTANT_POOL_TEXT)) {
        return constPoolTip;
      }
      else if(nodeText.equals(ACCESS_FLAGS_TEXT)) {
        return classFile.getAccessFlags().toString();
      }
      else if(nodeText.equals(THIS_CLASS_TEXT)) {
        return thisClassTip;
      }
      else if(nodeText.equals(SUPER_CLASS_TEXT)) {
        return superClassTip;
      }
      else if(nodeText.equals(INTERFACES_TEXT)) {
        return interfacesTip;
      }
      else if(nodeText.equals(FIELDS_TEXT)) {
        return fieldsTip;
      }
      else if(nodeText.equals(METHODS_TEXT)) {
        return methodsTip;
      }
      else if(nodeText.equals(CLASS_ATTRIBUTES_TEXT)) {
        return classAttrTip;
      }
      else if(nodeText.equals("Body")) {
        return bodyTip;
      }
      else if(nodeText.equals("ExceptionTable")) {
        return exceptionTableTip;
      }
      else if(nodeText.equals(AttributeEntry.LINE_NUMBER_TABLE)) {
        return lineNumbAttrTip;
      }
      else if(nodeText.equals(AttributeEntry.LOCAL_VARIABLE_TABLE)) {
        return localVarAttrTip;
      }
      else if(nodeText.equals(AttributeEntry.SYNTHETIC)) {
        return syntheticAttrTip;
      }
      else if(nodeText.equals(AttributeEntry.CONSTANT_VALUE)) {
        return constantValueAttrTip;
      }
      else if(nodeText.equals(ATTRIBUTES_TEXT)) {
        String text = node.getParent().toString();
        if(text.equals(AttributeEntry.CODE)) {
          return codeAttrTip;
        }
        else {
          text = node.getParent().getParent().toString();
          if(text.equals(FIELDS_TEXT)) {
            return fieldAttrTip;
          }
          else if(text.equals(METHODS_TEXT)) {
            return methodAttrTip;
          }
        }
      }
      else if(nodeText.equals(AttributeEntry.SOURCE_FILE)) {
        return sourceFileAttrTip;
      }
      else if(nodeText.equals(AttributeEntry.INNER_CLASSES)) {
        return innerClassesAttrTip;
      }
      else {
        String parentText = node.getParent().toString();
        if(parentText.equals(AttributeEntry.INNER_CLASSES)) {
          return innerClassTip;
        }
        else if(parentText.equals(AttributeEntry.SOURCE_FILE)) {
          return sourceFileTip;
        }
        else if(parentText.equals(FIELDS_TEXT)) {
          return fieldTip;
        }
        else if(parentText.equals(METHODS_TEXT)) {
          return methodTip;
        }
        else if(parentText.equals(ATTRIBUTES_TEXT)) {
          if(nodeText.equals(AttributeEntry.CODE)) {
            return codeTip;
          }
          else if(nodeText.equals(AttributeEntry.EXCEPTIONS)) {
            return exceptionsTip;
          }
        }
        else if(parentText.equals(AttributeEntry.EXCEPTIONS)) {
          return exceptionTip;
        }
        else if(parentText.equals("ExceptionTable")) {
          return exceptionHandlerTip;
        }
        else if(parentText.equals(AttributeEntry.LINE_NUMBER_TABLE)) {
          return lineNumbTip;
        }
        else if(parentText.equals(AttributeEntry.LOCAL_VARIABLE_TABLE)) {
          return localVarTip;
        }

        return nodeText;
      }
    }

    return null;
  }

  public String getMagicNumber() {
    return Converter.toHexString(classFile.getMagicNumber());
  }

  public String getMinorVersion() {
    return Converter.toHexString(classFile.getMinorVersion());
  }

  public String getMajorVersion() {
    return Converter.toHexString(classFile.getMajorVersion());
  }

  public String getAccessFlags() {
    return Converter.toHexString(classFile.getAccessFlags().getAccessFlags());
  }

  public String getThisClass() {
    return Converter.toHexString(classFile.getThisClass());
  }

  public String getSuperClass() {
    return Converter.toHexString(classFile.getSuperClass());
  }

  public String getThisName() {
    String name = null;
    PoolEntry entry  = constPool.getEntry(classFile.getThisClass());
    try {
      name  = (entry.resolve(constPool)[1]).replace('/', '.');
    }
    catch(Exception e) { e.printStackTrace(); }

    return name;
  }

  public String getSuperName() {
    String name = "";
    short superClass = classFile.getSuperClass();
    if(superClass != 0) {
      PoolEntry entry  = constPool.getEntry(superClass);
      try {
        name  = (entry.resolve(constPool)[1]).replace('/', '.');
      }
      catch(Exception e) { e.printStackTrace(); }
    }

    return name;
  }

  public String getHexSize(EntryCollection entries) {
    return Converter.toHexString((short)(entries.size()));
  }

  // implements TreeTableModel

  // Names of the columns.
  static protected String[] cNames = { "Section", "Binary image" };

  // Types of the columns.
  static protected Class[] cTypes = { TreeTableModel.class, String.class };

  public int getColumnCount() {
    return cNames.length;
  }

  public String getColumnName(int column) {
    return cNames[column];
  }

  public Class getColumnClass(int column) {
    return cTypes[column];
  }

  public Object getValueAt(Object o, int column) {
    TreeTableNode node = (TreeTableNode)o;

    switch(column) {
      case 0:
        return node.getText();
      case 1:
        return node.getInfo();
    }

    return null;
  }

  public boolean isCellEditable(Object node, int column) {
    return getColumnClass(column) == TreeTableModel.class;
  }

  public void setValueAt(Object aValue, Object node, int column) {}

}
