// BodyTable.java

package org.javalobby.cafebabe.plugin.classplug.bodyeditor;

import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.util.*;

import javax.swing.*;
import javax.swing.table.*;
import javax.swing.border.*;

import org.javalobby.classfile.*;
import org.javalobby.classfile.attribute.*;
import org.javalobby.classfile.instruction.*;
import org.javalobby.util.*;

import org.javalobby.cafebabe.*;
import org.javalobby.cafebabe.plugin.classplug.*;

public abstract class BodyTable extends JTable implements Opcode, ClipboardOwner {

  private BodyTableModel model;

  private JPopupMenu instructionMenu = new InstructionMenu();

  private ConstPool constPool;

  private Component parent;
  private MethodNode methodNode;
  private ClassFile classFile;

  public BodyTable(Component parent, MethodNode methodNode, ClassFile classFile) {
    super();

    this.parent     = parent;
    this.methodNode = methodNode;
    this.classFile  = classFile;

    constPool = classFile.getConstPool();

    setModel(new BodyTableModel(methodNode, constPool));

    model = (BodyTableModel)getModel();

    TableColumn column1 = getColumnModel().getColumn(0);
    TableColumn column2 = getColumnModel().getColumn(1);
    TableColumn column3 = getColumnModel().getColumn(2);
    column1.setPreferredWidth(45);
    column2.setPreferredWidth(140);
    column3.setPreferredWidth(600);

    TableCellRenderer renderer = new BodyRenderer();
    column3.setCellRenderer(renderer);

    setShowHorizontalLines(false);

    setRowSelectionInterval(0, 0);

    addMouseListener(new CustomMouseAdapter());

    ToolTipManager.sharedInstance().registerComponent(this);
    ToolTipManager.sharedInstance().setInitialDelay(2000);
  }

  // implements ClipboardOwner
  public void lostOwnership(Clipboard clipboard, Transferable contents) {}

  public String getToolTipText(MouseEvent e) {

    String toolTip = null;

    Point p = new Point(e.getX(), e.getY());
    int row    = this.rowAtPoint(p);
    int column = this.columnAtPoint(p);

    toolTip = ((BodyTableModel)getModel()).getTooltipAt(row, column);

    return toolTip;
  }

  public String getCurrentMnemonic() {
    int row = getSelectedRow();

    if(row == -1)
      return null;

    return (String)getValueAt(row, 1);
  }

  public void editInstruction() {
    int currentRow = (model.getRowCount() == 0) ? -1 : getSelectedRow();

    if(currentRow == -1) {
      JOptionPane.showMessageDialog(this.getTopLevelAncestor(),
                  "Please select an instruction for editing.\n",
                  "Warning", JOptionPane.WARNING_MESSAGE);
       return;
    }

    InstructionEditor instructionEditor = new InstructionEditor(parent, constPool);

    short offset = model.getOffset(currentRow);
    Instruction instruction = model.getInstruction(currentRow);

    instructionEditor.setInstruction(offset, instruction);

    instructionEditor.setTitle("Edit instruction");
    instructionEditor.show();

    if(instructionEditor.isOk()) {
      model.editInstruction(instructionEditor.getInstruction(), currentRow);
      updateClassFile();
    }
  }

  public void addInstruction(Instruction instruction) {
    int currentRow = getSelectedRow();

    if(currentRow == -1 && model.getRowCount() > 0) {
      JOptionPane.showMessageDialog(this.getTopLevelAncestor(),
                  "Please select an instruction after which " +
                  "new instruction(s) will be added.\n",
                  "Warning", JOptionPane.WARNING_MESSAGE);
       return;
    }

    InstructionEditor instructionEditor = new InstructionEditor(parent, constPool);

    short offset = (model.getRowCount() == 0) ? 0 : model.getOffset(currentRow);

    instructionEditor.setInstruction(offset, instruction);

    instructionEditor.setTitle("Add instruction");
    instructionEditor.show();

    if(instructionEditor.isOk()) {
      currentRow = (model.getRowCount() == 0) ? -1 : currentRow;
      model.addInstruction(instructionEditor.getInstruction(), currentRow+1);
      setRowSelectionInterval(currentRow+1, currentRow+1);

      updateClassFile();
    }
  }

  public boolean removeInstructions() {
    int[] rows = getSelectedRows();

    if(rows.length == 0 || model.getRowCount() == 0) {
      JOptionPane.showMessageDialog(this.getTopLevelAncestor(),
                  "Please select instruction(s) to remove.\n",
                  "Warning", JOptionPane.WARNING_MESSAGE);
      return false;
    }

    int value = JOptionPane.showConfirmDialog(this.getTopLevelAncestor(),
                    "Do you want to delete selected instruction(s)?",
                    "Confirmation",
                    JOptionPane.YES_NO_OPTION);

    if(value == JOptionPane.YES_OPTION) {
      Instruction[] instructions = new Instruction[rows.length];

      for(int i=0; i < rows.length; i++) {
        instructions[i] = model.getInstruction(rows[i]);
      }

      int row = (model.getRowCount() == 0) ? -1 : 0;
      model.removeInstructions(instructions);
      setRowSelectionInterval(0, 0);

      updateClassFile();
    }

    return true;
  }

  private void updateClassFile() {
    if(model.isBodyChanged()) {
      CodeAttribute codeAttribute = model.getCodeAttribute();

      byte[] buffer = BytesProducer.getBytes(codeAttribute);

      MethodEntry methodEntry = methodNode.getMethodEntry();
      AttributeEntry attribute =
                     methodEntry.getAttribute(AttributeEntry.CODE, constPool);

      attribute.setBuffer(buffer);

      classFile.setChanged(true);
    }
  }

  private Clipboard clipboard = new Clipboard("Instructions Clipboard");

  private void cutCommand() {
    int[] rows = getSelectedRows();

    if(rows.length == 0) {
      JOptionPane.showMessageDialog(this.getTopLevelAncestor(),
                  "Please select instruction(s).\n",
                  "Warning", JOptionPane.WARNING_MESSAGE);
       return;
    }

    Instruction[] instructions = new Instruction[rows.length];

    for(int i=0; i < rows.length; i++) {
      instructions[i] = model.getInstruction(rows[i]);
    }

    model.removeInstructions(instructions);

    clipboard.setContents(new InstructionsSelection(instructions), this);

    updateClassFile();
  }

  private void copyCommand() {
    int[] rows = getSelectedRows();

    if(rows.length == 0) {
      JOptionPane.showMessageDialog(this.getTopLevelAncestor(),
                  "Please select instruction(s).\n",
                  "Warning", JOptionPane.WARNING_MESSAGE);
       return;
    }

    Instruction[] instructions = new Instruction[rows.length];

    for(int i=0; i < rows.length; i++) {
      instructions[i] = model.getInstruction(rows[i]);
    }

    InstructionsSelection contents = new InstructionsSelection(instructions);

    clipboard.setContents(contents, this);
  }

  private void pasteCommand() {
    int currentRow = getSelectedRow();

    if(currentRow == -1) {
      JOptionPane.showMessageDialog(this.getTopLevelAncestor(),
                  "Please select an instruction after which " +
                  "new instruction(s) will be added.\n",
                  "Warning", JOptionPane.WARNING_MESSAGE);
       return;
    }

    Transferable content = clipboard.getContents(this);

    if(content != null) {
      try {
        Instruction[] instructions = (Instruction[])
           content.getTransferData(InstructionsSelection.instructionsFlavor);

        currentRow = (model.getRowCount() == 0) ? -1 : currentRow;

        for(int i=0; i < instructions.length; i++, currentRow++) {
          model.addInstruction(instructions[i], currentRow+1);
        }

        setRowSelectionInterval(currentRow+1, currentRow+1+instructions.length);

        if(instructions.length > 0) {
          updateClassFile();
        }
      }
      catch (Exception e) {
        getToolkit().beep();
      }
    }
  }

/*  public int getTotalColumnPreferredWidth() {
    Enumeration enum = getColumnModel().getColumns();
    int totalColumnWidth = 0;
    while (enum.hasMoreElements()) {
      int width =((TableColumn)enum.nextElement()).getPreferredWidth();
      totalColumnWidth += (width +getColumnModel().getColumnMargin());
    }
    return totalColumnWidth;
  }

  public boolean getScrollableTracksViewportWidth() {
    int viewPortWidth = getParent().getWidth();
    if(viewPortWidth < getTotalColumnPreferredWidth())
      return false; // scroll

    return !(autoResizeMode == AUTO_RESIZE_OFF);
  }
*/

class BodyRenderer extends DefaultTableCellRenderer implements ClassPlugConstants {

  private ImageIcon classIcon, fieldIcon, methodIcon, iMethodIcon,
                    stringIcon, condIcon, primitiveIcon;

  private boolean isSelected, hasFocus;
  private int column;

  BodyRenderer() {
    classIcon     = IconProducer.getImageIcon(ICON_CLASS);
    fieldIcon     = IconProducer.getImageIcon(ICON_FIELD);
    methodIcon    = IconProducer.getImageIcon(ICON_METHOD);
    iMethodIcon   = IconProducer.getImageIcon(ICON_IMETHOD);
    stringIcon    = IconProducer.getImageIcon(ICON_STRING);
    condIcon      = IconProducer.getImageIcon(ICON_CONDITION);
    primitiveIcon = IconProducer.getImageIcon(ICON_PRIMITIVE);
  }

  public Component getTableCellRendererComponent(
         JTable table, Object value, boolean isSelected,
         boolean hasFocus, int row, int column) {

    this.isSelected = isSelected;
    this.hasFocus   = hasFocus;
    this.column     = column;

    this.setEnabled(table.isEnabled());

    setIcon(null);

    String text = value.toString();
  //  System.out.println("text " + text);

    StringTokenizer st = new StringTokenizer(text);

    if(st.hasMoreTokens()) {
      String token = st.nextToken();

      int opcode = OpcodeTable.getOpcode((String)table.getValueAt(row, 1));
      if(opcode == Opcode.NEWARRAY) {
        setIcon(primitiveIcon);
      }
      else if(opcode == Opcode.TABLESWITCH || opcode == Opcode.LOOKUPSWITCH) {
        // do nothing
      }
      else if(!st.hasMoreTokens()) {
        if(OpcodeTable.isCondition((byte)opcode)) {
          setIcon(condIcon);

          OpcodeDescription description = OpcodeTable.getDescription((byte)opcode);
          int length = description.length-1;
          if(length == 1)
            text = Converter.toHexString(Byte.parseByte(text, 16));
          else if(length == 2)
            text = Converter.toHexString(Short.parseShort(text, 16));
          else if(length == 4)
            text = Converter.toHexString(Integer.parseInt(text, 16));
        }
        else {
          OpcodeDescription description = OpcodeTable.getDescription((byte)opcode);
          int lenght = description.length-1;
          if(lenght == 1)
            text = Converter.toHexString(Byte.parseByte(text, 16));
          else if(lenght == 2)
            text = Converter.toHexString(Short.parseShort(text, 16));
          else if(lenght == 4)
            text = Converter.toHexString(Integer.parseInt(text, 16));
        }
      }
      else {
        StringBuffer sb = new StringBuffer();
        sb.append(token);

        while(!token.equals("Const")) {
          token = st.nextToken();
          sb.append(" " + token);
        }

        String type = sb.toString();

        StringBuffer param = new StringBuffer();
        if(st.hasMoreTokens()) {
          param.append(st.nextToken());
        }

        while(st.hasMoreTokens()) {
          param.append(" ");
          param.append(st.nextToken());
        }

        if(type.startsWith(new ClassConst().getType())) {
          text = param.toString().replace('/', '.');
          setIcon(classIcon);
        }
        else if(type.startsWith(new FieldConst().getType())) {
          text = param.toString().replace('/', '.');
          setIcon(fieldIcon);
        }
        else if(type.startsWith(new MethodConst().getType())) {
          text = param.toString().replace('/', '.');
          setIcon(methodIcon);
        }
        else if(type.startsWith(new InterfaceMethodConst().getType())) {
          text = param.toString().replace('/', '.');
          setIcon(iMethodIcon);
        }
        else if(type.startsWith(new StringConst().getType())) {
          text = param.toString();
          setIcon(stringIcon);
        }
      }
    }

    return super.getTableCellRendererComponent(
                 table, text, isSelected, hasFocus, row, column);
  }

}

  public abstract void showTopic();

public class InstructionMenu extends JPopupMenu {
  final String CUT_STRING      = "Cut";
  final String COPY_STRING     = "Copy";
  final String PASTE_STRING    = "Paste";

  final String EDIT_STRING     = "Edit";
  final String ADD_STRING      = "Add";
  final String REMOVE_STRING   = "Remove";
  final String HELP_STRING     = "Help";

  public InstructionMenu() {
    super();

    JMenuItem cutItem    = new JMenuItem(CUT_STRING);
    JMenuItem copyItem   = new JMenuItem(COPY_STRING);
    JMenuItem pasteItem  = new JMenuItem(PASTE_STRING);

    JMenuItem editItem   = new JMenuItem(EDIT_STRING);
    JMenuItem addItem    = new JMenuItem(ADD_STRING);
    JMenuItem removeItem = new JMenuItem(REMOVE_STRING);
    JMenuItem helpItem   = new JMenuItem(HELP_STRING);

    ActionListener actionListener = new GenericActionListener();

    cutItem.addActionListener(actionListener);
    copyItem.addActionListener(actionListener);
    pasteItem.addActionListener(actionListener);

    editItem.addActionListener(actionListener);
    addItem.addActionListener(actionListener);
    removeItem.addActionListener(actionListener);
    helpItem.addActionListener(actionListener);

    this.add(cutItem);
    this.add(copyItem);
    this.add(pasteItem);
    this.addSeparator();
    this.add(editItem);
    this.add(addItem);
    this.add(removeItem);
    this.addSeparator();
    this.add(helpItem);
  }

class GenericActionListener implements ActionListener {

  public void actionPerformed(ActionEvent event) {
    String cmd = event.getActionCommand();
    if(cmd.equals(CUT_STRING)) {
      cutCommand();
    }
    else if(cmd.equals(COPY_STRING)) {
      copyCommand();
    }
    else if(cmd.equals(PASTE_STRING)) {
      pasteCommand();
    }
    else if(cmd.equals(EDIT_STRING)) {
      editInstruction();
    }
    else if(cmd.equals(ADD_STRING)) {
      addInstruction(null);
    }
    else if(cmd.equals(REMOVE_STRING)) {
      removeInstructions();
    }
    else if(cmd.equals(HELP_STRING)) {
      showTopic();
    }
  }

}

}

class CustomMouseAdapter extends MouseAdapter {

  public void mouseReleased(MouseEvent event) {
    if(event.isPopupTrigger()) {
      instructionMenu.show(event.getComponent(), event.getX(), event.getY());
    }
  }

}


}
