// HelpTree.java

package org.javalobby.cafebabe.plugin.classplug.help;

import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import javax.swing.border.*;

import org.javalobby.util.*;
import org.javalobby.cafebabe.*;

import org.javalobby.cafebabe.plugin.classplug.*;

public class HelpTree extends JTree {

  private RootNode root;
  private String helpDirectory;

  public HelpTree() {
    super();

    this.setEditable(false);
    this.setShowsRootHandles(false);
    this.setRootVisible(false);
    this.setBorder(new BevelBorder(BevelBorder.RAISED));
    this.setToolTipText("");

    // Create a new tree control
    HelpTreeModel treeModel = new HelpTreeModel();
    this.setModel(treeModel);

    root = (RootNode)treeModel.getRoot();

    setCellRenderer(new HelpTreeRenderer());

    ToolTipManager.sharedInstance().registerComponent(this);
  }

  public HelpTree(String helpIni) throws IOException {
    this();

    fillTopics(helpIni);
  }

  private void fillTopics(String helpIni) throws IOException {
    File fileProp = new File(helpIni);

    if(!fileProp.exists())
      throw new IOException("Properties file: <" + fileProp + ">" +
                            " doesn't exist.");

    IniFile iniFile = new IniFile(helpIni);

    helpDirectory = iniFile.getValue("directory", "directory");

    String[] topics = iniFile.getVariables("topics");
    for (int i = 0; i < topics.length; i++) {
      String topicName = topics[i];
      String refName   = iniFile.getValue("topics", topics[i]);

      String name = new File(helpDirectory + "/" + refName).getAbsolutePath();
      HelpLeafNode helpNode =
                   new HelpLeafNode(topicName, name.replace('\\', '/'));
      root.add(helpNode);
    }

    expandPath(new TreePath(root.getPath()));
  }


  public String getToolTipText(MouseEvent event) {
    int row = getRowForLocation(event.getX(), event.getY());
    if(row == -1) return null;

    TreePath path = getPathForLocation(event.getX(), event.getY());
    DefaultMutableTreeNode node =
           (DefaultMutableTreeNode)path.getLastPathComponent();

    return node.getUserObject().toString();
  }

  public Point getToolTipLocation(MouseEvent event) {
    int row       = getRowForLocation(event.getX(), event.getY());
    TreePath path = getPathForLocation(event.getX(), event.getY());

    if(row == -1) return null;
    if(path == null) return null;

    TreeNode node = (TreeNode)path.getLastPathComponent();
    if(node == null || node.toString().equals(""))
      return null;

    TreeCellRenderer renderer = getCellRenderer();
    Component comp = renderer.getTreeCellRendererComponent(
                              this, node, isRowSelected(row),
                              isExpanded(row), node.isLeaf(),
                              row, true);

    Point pt = getRowBounds(row).getLocation();
    if(pt == null)
      return null;

    pt.translate(20, 0);

    return pt;
  }


class HelpTreeRenderer extends JLabel implements TreeCellRenderer {

  private ImageIcon folderOpenIcon, folderClosedIcon;
  private ImageIcon helpIcon;

  private boolean isSelected, hasFocus;

  HelpTreeRenderer() {
    folderOpenIcon = IconProducer.getImageIcon(Constants.ICON_OPEN_FOLDER);
    folderClosedIcon = IconProducer.getImageIcon(Constants.ICON_CLOSED_FOLDER);

    helpIcon = IconProducer.getImageIcon(ClassPlugConstants.ICON_HELP);
  }

  public Component getTreeCellRendererComponent(JTree tree,
                   Object value, boolean isSelected, boolean expanded,
                   boolean leaf, int row, boolean hasFocus) {

    this.isSelected = isSelected;
    this.hasFocus   = hasFocus;

    String  stringValue = tree.convertValueToText(value, isSelected,
                        expanded, leaf, row, hasFocus);
    this.setEnabled(tree.isEnabled());

    setText(stringValue);

    DefaultMutableTreeNode node = (DefaultMutableTreeNode)value;

    if(!node.getAllowsChildren()) {
      if(node instanceof HelpLeafNode) {
        setIcon(helpIcon);
      }
      else {
        setIcon(UIManager.getIcon("Tree.leafIcon"));
      }
    }
    else {
      if(node instanceof RootNode) {
        setIcon(null);
      }
      else {
        if(expanded) {
          setIcon(folderOpenIcon);
        }
        else {
          setIcon(folderClosedIcon);
        }
      }
    }

    return this;
  }

  public void paint(Graphics g) {
    g.setFont(UIManager.getFont("Tree.font"));
    String str;
    if ((str = getText()) != null) {
      if (0 < str.length()) {
        if (isSelected) {
          g.setColor(UIManager.getColor("Tree.selectionBackground"));
        } else {
          g.setColor(UIManager.getColor("Tree.textBackground"));
        }
        Dimension d = this.getPreferredSize();
        int imageOffset = 0;
        Icon currentI = getIcon();
        if (currentI != null) {
          imageOffset = currentI.getIconWidth() + Math.max(0, getIconTextGap() - 1);
        }
        g.fillRect(imageOffset, 0, d.width -1 - imageOffset, d.height);
        if (hasFocus) {
          g.setColor(UIManager.getColor("Tree.selectionBorderColor"));
          g.drawRect(imageOffset, 0, d.width -1 - imageOffset, d.height -1);
       }
      }
    }
    super.paint(g);
  }

}

}
