// ClassHoundSingleton.java

package org.javalobby.cafebabe.plugin.houndplug;

import java.io.*;
import java.util.*;
import java.util.zip.*;

import org.javalobby.classfile.*;

public class ClassHoundSingleton {

  public final static String ANONYMOUS_PACKAGE = "<anonymous package>";

  private final Vector entries      = new Vector();
  private final Vector packages     = new Vector();
  private final Vector classes      = new Vector();
  private final Vector fields       = new Vector();
  private final Vector methods      = new Vector();

  // map: classpath entry -> files list
  private final Hashtable entryToFilesMap = new Hashtable();

  private static ClassHoundSingleton instance;

  private ClassHoundSingleton() {
    init();
  }

  public static ClassHoundSingleton getInstance() {
    if(instance == null)
      instance = new ClassHoundSingleton();

    return instance;
  }

  private void init() {
    collectDefaults(entries);

    int size = entries.size();
    for(int i=0; i < size; i++)
      addEntry((String)entries.elementAt(i));
  }

  public void collectDefaults(Vector entriesList) {
    StringBuffer defaults = new StringBuffer();
    if(System.getProperty("java.version").startsWith("1.2")) {
      String javaHome = System.getProperty("java.home");
      String libHome  = javaHome + File.separator + "lib";
      String[] list = (new File(libHome)).list();
      if(list != null) {
        for(int i=0; i < list.length; i++) {
          String name = list[i].toLowerCase();
          if(name.endsWith("jar") || name.endsWith("zip"))
            defaults.append(libHome + File.separator + list[i] + File.pathSeparator);
        }
      }

      String extHome = libHome + File.separator + "ext";
      list = (new File(extHome)).list();
      if(list != null) {
        for(int i=0; i < list.length; i++) {
          String name = list[i].toLowerCase();
          if(name.endsWith("jar") || name.endsWith("zip"))
            defaults.append(extHome + File.separator + list[i] + File.pathSeparator);
        }
      }

      String toolHome = javaHome + File.separator + ".." + File.separator + "lib";
      list = (new File(toolHome)).list();
      if(list != null) {
        for(int i=0; i < list.length; i++) {
          String name = list[i].toLowerCase();
          if(name.endsWith("jar") || name.endsWith("zip"))
            defaults.append(toolHome + File.separator + list[i] + File.pathSeparator);
        }
      }
    }

    defaults.append(System.getProperty("java.class.path"));

    StringTokenizer st = new StringTokenizer(defaults.toString(), File.pathSeparator);
    while(st.hasMoreTokens()) {
      String entryName = (String)st.nextToken();
      File fileEntry = new File(entryName);
      if(!fileEntry.exists())
        continue;

      String name = null;
      try {
        name = fileEntry.getCanonicalPath();
        if(!entriesList.contains(name)) {
          entriesList.addElement(name);
        }
      }
      catch (IOException e2) {
        e2.printStackTrace();
      }
    }
  }

  public void addEntry(String entryName) {
    Vector filesForEntry = new Vector();

    File file = new File(entryName);
    if(file.isDirectory())
      collectFromDirectory(entryName, "", filesForEntry);
    else
      collectFromArchive(entryName, filesForEntry);

    entryToFilesMap.put(entryName, filesForEntry);
  }

  public void removeEntry(String entryName) {
    entryToFilesMap.remove(entryName);
  }


  private void collectFromDirectory(String parent, String child, Vector list) {
    String dirName = (child.length() == 0) ? parent : parent + File.separator + child;

    String[] files = new File(dirName).list();
    for(int i=0; i < files.length; i++) {
      String shortName = (child.length() == 0) ? files[i] : child + File.separator + files[i];
      String fullName  = dirName + File.separator + files[i];

      if(new File(fullName).isDirectory())
        collectFromDirectory(parent, shortName, list);
      else if(shortName.toLowerCase().endsWith(".class"))
        list.addElement(shortName);
    }
  }

  private void collectFromArchive(String archiveName, Vector list) {
    try {
      ZipFile zipFile = new ZipFile(archiveName);
      for(Enumeration e = zipFile.entries(); e.hasMoreElements(); ) {
        ZipEntry entry = (ZipEntry)e.nextElement();
        String name = entry.getName();
        if(name.toLowerCase().endsWith(".class")) {
          list.addElement(entry.getName());
        }
      }
    }
    catch(Exception e) {}
  }

  public Vector getEntries() { return entries; }

  public Vector getPackages() { return packages; }

  public Vector getClasses() { return classes; }

  public Vector getFields() { return fields; }

  public Vector getMethods() { return methods; }

  public Hashtable getEntryToFilesMap() { return entryToFilesMap; }


  public void clearPackages() {
    packages.removeAllElements();
  }

  public void clearClasses() {
    classes.removeAllElements();
  }

  public void clearMembers() {
    fields.removeAllElements();
    methods.removeAllElements();
  }

  public void fillPackagesList(String selectedEntry) {
    clearPackages();
    clearClasses();
    clearMembers();

    boolean isArchive   = !(new File(selectedEntry)).isDirectory();

    Vector files = (Vector)entryToFilesMap.get(selectedEntry);
    if(files == null) return;

    for(int i=0; i < files.size(); i++) {
      String fileName = (String)files.elementAt(i);
      String packageName = null;

      int pos = 0;
      if(isArchive)
        pos = fileName.lastIndexOf("/");
      else
        pos = fileName.lastIndexOf(File.separator);

      if(pos == -1)
        packageName = ANONYMOUS_PACKAGE;
      else {
        packageName = fileName.substring(0, pos).replace(File.separatorChar, '.');
        if(isArchive)
          packageName = packageName.replace('/', '.');
      }

      if(!packages.contains(packageName))
        packages.addElement(packageName);
    }

    sort(packages);
  }

  public void fillClassesList(String selectedEntry, String selectedPackage) {
    if(selectedPackage == null) return;

    clearClasses();
    clearMembers();

    boolean isArchive   = !(new File(selectedEntry)).isDirectory();

    Vector files = (Vector)entryToFilesMap.get(selectedEntry);
    if(files == null) return;

    for(int i=0; i < files.size(); i++) {
      String fileName = (String)files.elementAt(i);

      String className   = null,
             packagePart = null;

      int pos1 = 0;
      if(isArchive)
        pos1 = fileName.lastIndexOf("/");
      else
        pos1 = fileName.lastIndexOf(File.separator);

      int pos2 = fileName.lastIndexOf(".");
      if(pos1 == -1)
        className = fileName.substring(0, pos2);
      else {
        packagePart = fileName.substring(0, pos1).replace(File.separatorChar, '.');
        if(isArchive)
          packagePart = packagePart.replace('/', '.');
        className   = fileName.substring(pos1+1, pos2);
      }

      if(packagePart == null) {
        if(selectedPackage.equals(ANONYMOUS_PACKAGE))
          if(!classes.contains(className))
            classes.addElement(className);
      }
      else {
        if(packagePart.equals(selectedPackage))
          if(!classes.contains(className))
            classes.addElement(className);
      }
    }

    sort(classes);
  }

  public void fillMembersLists(String selectedEntry, String selectedPackage,
                                String selectedClass) {
    if(selectedClass == null) return;

    clearMembers();

    try {
      String fromFile = selectedClass + ".class";
      if(selectedPackage != ANONYMOUS_PACKAGE)
        fromFile = selectedPackage.replace('.', '/') + "/" + fromFile;

      InputStream is = null;
      if((new File(selectedEntry)).isDirectory()) {
        is = new FileInputStream(selectedEntry + File.separator + fromFile);
      }
      else {
        ZipFile zipFile = new ZipFile(selectedEntry);
        ZipEntry zipEntry = zipFile.getEntry(fromFile);
        if(zipEntry == null) return;
        is = zipFile.getInputStream(zipEntry);
      }

      BufferedInputStream bis = new BufferedInputStream(is);
      ClassFile classFile = new ClassFile();
      classFile.read(bis);

      ConstPool constPool = classFile.getConstPool();
      EntryCollection fieldsCollection  = classFile.fields();
      EntryCollection methodsCollection = classFile.methods();

      for(short i=0; i < fieldsCollection.size(); i++) {
        FieldEntry entry = (FieldEntry)fieldsCollection.elementAt(i);
        String[] str = entry.resolve(constPool);
        fields.addElement(Converter.merge(str, " "));
      }
      sort(fields);

      for(short i=0; i < methodsCollection.size(); i++) {
        MethodEntry entry = (MethodEntry)methodsCollection.elementAt(i);
        String[] str = entry.resolve(constPool);
        methods.addElement(Converter.merge(str, " "));
      }
      sort(methods);
    }
    catch(Exception e) { e.printStackTrace(); }
  }

  public static void sort(Vector vector) {
    int i = vector.size();
    boolean flag = true;
    while(flag) {
      flag = false;
      for(int k = 0; k < i - 1; k++) {
        String el1 = (String)vector.elementAt(k);
        String el2 = (String)vector.elementAt(k + 1);
        if(el1.compareTo(el2) > 0) {
          String s = (String)vector.elementAt(k + 1);
          vector.removeElementAt(k + 1);
          vector.insertElementAt(s, k);
          flag = true;
        }
      }
    }
  }

}
