// AccessFlags.java

package org.javalobby.classfile;

import java.io.*;

/**
 * Class that represents access modifiers for class members
 * (fields, methods, class attributes)
 *
 * @version 1.0 02/22/2000
 * @author Alexander Shvets
 */
public final class AccessFlags implements Entry {
  /** public flag */
  public static final int ACC_PUBLIC       = 1;

  /** private flag */
  public static final int ACC_PRIVATE      = 2;

  /** protected flag */
  public static final int ACC_PROTECTED    = 4;

  /** static flag */
  public static final int ACC_STATIC       = 8;

  /** final flag */
  public static final int ACC_FINAL        = 16;

  /** synchronized flag */
  public static final int ACC_SYNCHRONIZED = 32;

  /** volatile flag */
  public static final int ACC_VOLATILE     = 64;

  /** transient flag */
  public static final int ACC_TRANSIENT    = 128;

  /** native flag */
  public static final int ACC_NATIVE       = 256;

  /** interface flag */
  public static final int ACC_INTERFACE    = 512;

  /** abstract flag */
  public static final int ACC_ABSTRACT     = 1024;

  /* a short value for holding all attributes */
  private short accessFlags;

  /**
   * The default constructor that creates "access flags" entry.
   */
  public AccessFlags() {}

  /**
   * Read this entry from input stream.
   *
   * @param  in  input stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void read(DataInput in) throws IOException {
    accessFlags = in.readShort();
  }

  /**
   * Write this entry to output stream.
   *
   * @param  out  output stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void write(DataOutput out) throws IOException {
    out.writeShort(accessFlags);
  }

  /**
   * Get type of this entry.
   *
   * @return  type of entry
   */
  public String getType() {
    return "Access Flags";
  }

  /**
   * Length of this entry in bytes.
   *
   * @return  length of entry
   */
  public long length() {
    return 2;
  }

  /**
   * Get access flags.
   *
   * @return access flags
   */
  public short getAccessFlags() {
    return accessFlags;
  }

  /**
   * set access flags.
   *
   * @param accessFlags access flags
   */
  public void setAccessFlags(short accessFlags) {
    this.accessFlags = accessFlags;
  }

  /**
   * Compares two access flags consts
   * will be equals if they both have the same flags.
   *
   * @param   f  the flag tor comparison.
   * @return  true if this flag has the same flags; false otherwise.
   */
  public boolean equals(AccessFlags f) {
    return (accessFlags == f.accessFlags);
  }

  /**
   * Check if "public" flag is on
   *
   * @return true if "public" flag is on; false otherwise
   */
  public boolean isPublic() {
    return (accessFlags & ACC_PUBLIC) != 0;
  }

  /**
   * Check if "private" flag is on
   *
   * @return true if "private" flag is on; false otherwise
   */
  public boolean isPrivate() {
    return (accessFlags & ACC_PRIVATE) != 0;
  }

  /**
   * Check if "protected" flag is on
   *
   * @return true if "protected" flag is on; false otherwise
   */
  public boolean isProtected() {
    return (accessFlags & ACC_PROTECTED) != 0;
  }

  /**
   * Check if "final" flag is on
   *
   * @return true if "final" flag is on; false otherwise
   */
  public boolean isFinal() {
    return (accessFlags & ACC_FINAL) != 0;
  }

  /**
   * Check if "synchronized" flag is on
   *
   * @return true if "synchronized" flag is on; false otherwise
   */
  public boolean isSynchronized() {
    return (accessFlags & ACC_SYNCHRONIZED) != 0;
  }

  /**
   * Check if "volatile" flag is on
   *
   * @return true if "volatile" flag is on; false otherwise
   */
  public boolean isVolatile() {
    return (accessFlags & ACC_VOLATILE) != 0;
  }

  /**
   * Check if "transient" flag is on
   *
   * @return true if "transient" flag is on; false otherwise
   */
  public boolean isTransient() {
    return (accessFlags & ACC_TRANSIENT) != 0;
  }

  /**
   * Check if "native" flag is on
   *
   * @return true if "native" flag is on; false otherwise
   */
  public boolean isNative() {
    return (accessFlags & ACC_NATIVE) != 0;
  }

  /**
   * Check if "interface" flag is on
   *
   * @return true if "interface" flag is on; false otherwise
   */
  public boolean isInterface() {
    return (accessFlags & ACC_INTERFACE) != 0;
  }

  /**
   * Check if "abstract" flag is on
   *
   * @return true if "abstract" flag is on; false otherwise
   */
  public boolean isAbstract() {
    return (accessFlags & ACC_ABSTRACT) != 0;
  }

  /**
   * Returns a string representation of the object.
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    StringBuffer sb = new StringBuffer();

    if((accessFlags & 0x07) == 0)              sb.append("package");
    else if((accessFlags & ACC_PUBLIC) > 0)    sb.append("public");
    else if((accessFlags & ACC_PRIVATE) > 0)   sb.append("private");
    else if((accessFlags & ACC_PROTECTED) > 0) sb.append("protected");

    if((accessFlags & ACC_STATIC) > 0)         sb.append(" static");
    if((accessFlags & ACC_FINAL) > 0)          sb.append(" final");
    if((accessFlags & ACC_SYNCHRONIZED) > 0)   sb.append(" synchronized");
    if((accessFlags & ACC_VOLATILE) > 0)       sb.append(" volatile");
    if((accessFlags & ACC_TRANSIENT) > 0)      sb.append(" transient");
    if((accessFlags & ACC_NATIVE) > 0)         sb.append(" native");
    if((accessFlags & ACC_ABSTRACT) > 0)       sb.append(" abstract");
    if((accessFlags & ACC_INTERFACE) > 0)      sb.append(" interface");

    return sb.toString();
  }

}
