// AttributeEntry.java

package org.javalobby.classfile;

import java.io.*;
import java.util.*;

// class that represent attribute entry (class, method or field attribute)

public class AttributeEntry implements Entry {

  /** A constant for presentation the name of "Source File" attribute */
  public static final String SOURCE_FILE          = "SourceFile";

  /** A constant for presentation the name of "Code" attribute */
  public static final String CODE                 = "Code";

  /** A constant for presentation the name of "Exceptions" attribute */
  public static final String EXCEPTIONS           = "Exceptions";

  /** A constant for presentation the name of "Line Number Table" attribute */
  public static final String LINE_NUMBER_TABLE    = "LineNumberTable";

  /** A constant for presentation the name of "Local Variable Table" attribute */
  public static final String LOCAL_VARIABLE_TABLE = "LocalVariableTable";

  /** A constant for presentation the name of "Inner Classes" attribute */
  public static final String INNER_CLASSES        = "InnerClasses";

  /** A constant for presentation the name of "Constant Value" attribute */
  public static final String CONSTANT_VALUE       = "ConstantValue";

  /** A constant for presentation the name of "Synthetic" attribute */
  public static final String SYNTHETIC            = "Synthetic";

  /** A constant for presentation the name of "Deprecated" attribute */
  public static final String DEPRECATED           = "Deprecated";

  /** the index value that points to the name of this entry */
  protected short nameIndex;

  /** An array of bytes that holds all information attached to this attribute entry.
   *  This information should be resolved lately. */
  protected byte buffer[];

  /**
   * The default constructor that creates attribute entry.
   */
  public AttributeEntry() {}

  /**
   * Read this entry from input stream.
   *
   * @param  in  input stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void read(DataInput in) throws IOException {
    nameIndex = in.readShort();
    int size = in.readInt();
    buffer  = new byte[size];
    in.readFully(buffer);
  }

  /**
   * Write this entry to output stream.
   *
   * @param  out  output stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void write(DataOutput out) throws IOException {
    int size = buffer.length;

    out.writeShort(nameIndex);
    out.writeInt(size);
    out.write(buffer, 0, size);
  }

  /**
   * Get type of this entry.
   *
   * @return  type of entry
   */
  public String getType() {
    return "Attribute";
  }

  /**
   * Length of this entry in bytes
   * (name index size + size of buffer size + number of elements in the buffer).
   *
   * @return  length of entry
   */
  public long length() {
    return 2 + 4 + buffer.length;
  }

  /**
   * Get the index that points to the name of this entry
   *
   * @return  the index that points to the name of this entry
   */
  public short getNameIndex() {
    return nameIndex;
  }

  /**
   * Get the byte array with a whole content of this attribute.
   *
   * @return  the byte array with a whole content of this attribute
   */
  public byte[] getBuffer() {
    return buffer;
  }

  /**
   * Set the byte array for this attribute.
   *
   * @param buffer  the byte array
   */
  public void setBuffer(byte[] buffer) {
    this.buffer = buffer;
  }

  /**
   * Get the name of this attribute
   *
   * @return  the name of this attribute
   */
  public String getName(Pool constPool) {
    return ((Utf8Const)constPool.getEntry(nameIndex)).getValue();
  }

  /**
   * Returns a string representation of the object.
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    int size = buffer.length;
    StringBuffer sb = new StringBuffer();
    for(int i=0; i < size; i++)
      sb.append(buffer[i] + " ");

    return "<" + nameIndex + " [(" + size + ") " + sb.toString() + "]>";
  }

}

