// ClassFile.java

package org.javalobby.classfile;

import java.io.*;
import java.util.*;
import java.util.zip.*;

import org.javalobby.classfile.attribute.*;

// class that represent single class file

public class ClassFile {

  public static final int JAVA_MAGIC         = 0xCAFEBABE;
  public static final int JAVA_VERSION       = 45;
  public static final int JAVA_MINOR_VERSION = 3;

  protected int magicNumber;
  protected short minorVersion, majorVersion;

  protected ConstPool constPool     = new ConstPool();
  protected AccessFlags accessFlags = new AccessFlags();

  protected short thisClass, superClass;

  protected EntryCollection interfaces =
                            new EntryCollection(InterfaceEntry.class, "Interfaces");
  protected EntryCollection fields =
                            new EntryCollection(FieldEntry.class, "Fields");
  protected EntryCollection methods =
                            new EntryCollection(MethodEntry.class, "Methods");
  protected EntryCollection attributes =
                            new EntryCollection(AttributeEntry.class, "Attributes");

  // for checking some modification of class file
  private boolean changed = false;

  public long length() {
    return 4 + 2 + 2 +              // CAFEBABE, Minor version, Major version
           2 + constPool.length() +
           2 + 2 + 2 +
           interfaces.length() +
           fields.length() +
           methods.length() +
           attributes.length();
  }

  public synchronized void read(String className) throws IOException {
    FileInputStream fis = new FileInputStream(className);
    read(fis);
  }

  public synchronized void read(InputStream is) throws IOException {
    DataInputStream in = new DataInputStream(
                             new BufferedInputStream(is, 8192));
    magicNumber = in.readInt();

    if(magicNumber != JAVA_MAGIC) {
      throw new IOException("Illegal magic number '0x" +
                             Integer.toHexString(magicNumber) + "'");
    }

    minorVersion = in.readShort();
    majorVersion = in.readShort();

    constPool.read(in);

    accessFlags.read(in);

    thisClass    = in.readShort();
    superClass   = in.readShort();

    interfaces.read(in);
    fields.read(in);
    methods.read(in);
    attributes.read(in);

    changed = false;

    in.close();
  }

  public synchronized void write(String fileName) throws IOException {
    write(new FileOutputStream(fileName));
  }

  public synchronized void write(OutputStream os) throws IOException {
    DataOutputStream out = new DataOutputStream(
                               new BufferedOutputStream(os, 8192));

    out.writeInt(magicNumber);

    out.writeShort(minorVersion);
    out.writeShort(majorVersion);

    constPool.write(out);

    accessFlags.write(out);

    out.writeShort(thisClass);
    out.writeShort(superClass);

    interfaces.write(out);
    fields.write(out);
    methods.write(out);
    attributes.write(out);

    changed = false;

    out.close();
  }

  public boolean isChanged() {
    return changed;
  }

  public void setChanged(boolean changed) {
    this.changed = changed;
  }

  public int getMagicNumber() {
    return magicNumber;
  }

  public short getMinorVersion() {
    return minorVersion;
  }

  public short getMajorVersion() {
    return majorVersion;
  }

  public short getThisClass() {
    return thisClass;
  }

  public short getSuperClass() {
    return superClass;
  }

  public AccessFlags getAccessFlags() {
    return accessFlags;
  }

  public ConstPool getConstPool() {
    return constPool;
  }

  public EntryCollection constants() {
   return constPool;
  }

  public EntryCollection interfaces() {
    return interfaces;
  }

  public EntryCollection fields() {
    return fields;
  }

  public EntryCollection methods() {
    return methods;
  }

  public EntryCollection attributes() {
    return attributes;
  }

  public String getClassName() {
    ClassConst classConst = (ClassConst)constPool.getEntry(thisClass);
    short index = classConst.getNameIndex();

    return ((Utf8Const)constPool.getEntry(index)).getValue();
  }

  public String getSuperclassName() {
    ClassConst classConst = (ClassConst)constPool.getEntry(superClass);
    short index = classConst.getNameIndex();

    return ((Utf8Const)constPool.getEntry(index)).getValue();
  }

  public String[] getInterfacesNames() {
    short len = (short)interfaces.size();

    String[] iii = new String[len];

    for (short i=0; i < len; i++) {
      InterfaceEntry entry = (InterfaceEntry)interfaces.elementAt(i);
      ClassConst entry2    = (ClassConst)constPool.getEntry(entry.getNameIndex());
      Utf8Const utf        = (Utf8Const)constPool.getEntry(entry2.getNameIndex());
      iii[i]               = utf.getValue();
    }

    return iii;
  }

  public void print(String fileName) throws IOException {
    PrintWriter out = new PrintWriter(new FileWriter(fileName));

    out.println();
    out.println("Magic number  : " + Converter.toHexString(magicNumber));
    out.println("Minor version : " + Converter.toHexString(minorVersion));
    out.println("Major version : " + Converter.toHexString(majorVersion));

    out.println("\nConstant Pool : " + Converter.toHexString((short)constPool.size()));

    for(short i=1; i < constPool.size(); i++) {
      PoolEntry constant = (PoolEntry)constPool.elementAt(i);
      if(constant == null) continue;
      out.print(Converter.toHexString(i, "") + '\t');
      String[] str = constant.resolve(constPool);
      for(int j=0; j < str.length; j++) {
        out.print(str[j] + " ");
      }
      out.println();
    }

    out.print("\nAccess flags : " + getAccessFlags() + "\t" + accessFlags);

    PoolEntry thisClassEntry  = constPool.getEntry(thisClass);
    PoolEntry superClassEntry = constPool.getEntry(superClass);

    out.print("\n\nThis class   : " + getThisClass() + " " +
              thisClassEntry.resolve(constPool)[1]);
    out.print("\n\nSuper class  : " + getSuperClass() + " " +
              superClassEntry.resolve(constPool)[1]);

    out.println("\n\nInterfaces: " + Converter.toHexString((short)interfaces.size(), ""));
    for(short i=0; i < interfaces.size(); i++) {
      InterfaceEntry entry = (InterfaceEntry)interfaces.elementAt(i);
      out.print(Converter.toHexString((short)(i+1)) + '\t');
      printEntry(entry, out);
    }

    out.println("\nFields: " + Converter.toHexString((short)fields.size(), ""));
    for(short i=0; i < fields.size(); i++) {
      FieldEntry entry = (FieldEntry)fields.elementAt(i);
      out.print(Converter.toHexString((short)(i+1)) + '\t');
      printEntry(entry, out);
    }

    out.println("\nMethods: " + Converter.toHexString((short)methods.size(), ""));
    for(short i=0; i < methods.size(); i++) {
      MethodEntry entry = (MethodEntry)methods.elementAt(i);
      out.print(Converter.toHexString((short)(i+1)) + '\t');
      printEntry(entry, out);
    }

    out.println("\nClass attributes: " + Converter.toHexString((short)attributes.size(), ""));
    for(short i=0; i < attributes.size(); i++) {
      AttributeEntry entry = (AttributeEntry)attributes.elementAt(i);
      out.print(Converter.toHexString((short)(i+1)) + '\t');
      printEntry(entry, out);
    }

    out.close();
  }

  private void printEntry(InterfaceEntry entry, PrintWriter out) {
    try {
      String[] str = entry.resolve(constPool);
      out.print(str[0] + '\t');
/*      for(int i=0; i < str[1].length; i++)
        out.print(str[1][i] + " ");
*/
    }
    catch(Exception e) {
      e.printStackTrace();
    }
    out.println();
  }


  private void printEntry(MemberEntry entry, PrintWriter out) {
    try {
      String[] str = entry.resolve(constPool);

      out.print(str[0]);
      for(int i=1; i < str.length; i++) out.print(" " + str[i]);
//      out.print(" " + str[1][0]);
//      for(int i=1; i < str[1].length; i++) out.print(" " + str[1][i]);

      EntryCollection attributes1 = entry.getAttributes();

      out.print(" " + Converter.toHexString((short)attributes1.size()));

      out.println();

      for(short i=0;  i < attributes1.size(); i++) {
        AttributeEntry entry1 = (AttributeEntry)attributes1.elementAt(i);
//        out.print("" + i + '\t');
        printEntry(entry1, out);
      }
    }
    catch(Exception e) {
      e.printStackTrace();
    }
    out.println();
  }


  private void printEntry(AttributeEntry attribute, PrintWriter out) {
    String attrName = attribute.getName(constPool);

    if(attrName.equals(AttributeEntry.SOURCE_FILE)) {
//      printSourceFileAttribute(attribute);
    }
    else if(attrName.equals(AttributeEntry.CONSTANT_VALUE)) {
//      attribute = new ConstantValueAttribute(attribute);
    }
    else if(attrName.equals(AttributeEntry.INNER_CLASSES)) {
//      attribute = new InnerClassesAttribute(attribute);
    }
    else if(attrName.equals(AttributeEntry.EXCEPTIONS)) {
//      attribute = new ExceptionsAttribute(attribute);
    }
    else if(attrName.equals(AttributeEntry.CODE)) {
      printCodeAttribute(new CodeAttribute(attribute), out);
    }
    else if(attrName.equals(AttributeEntry.LINE_NUMBER_TABLE)) {
//      insert(folder, new LineNumberTableAttribute(attribute));
    }
    else if(attrName.equals(AttributeEntry.LOCAL_VARIABLE_TABLE)) {
//      insert(folder, new LocalVariableTableAttribute(attribute));
    }
  }

  private void printCodeAttribute(CodeAttribute attribute, PrintWriter out) {
    out.println("Code" + " " + Converter.toHexString(attribute.getNameIndex()) + " " +
              Converter.toHexString(attribute.getBuffer().length));

    out.println("  maxStack  = " + Converter.toHexString(attribute.getMaxStack()));
    out.println("  maxLocals = " + Converter.toHexString(attribute.getMaxLocals()));

    MethodBody body = attribute.getMethodBody();

    out.println("  Body " + Converter.toHexString(body.length()));

    ExceptionHandlerEntry[] exceptionHandlers = attribute.getExceptionHandlers();

    out.println("  ExceptionTable " + Converter.toHexString((short)exceptionHandlers.length));

    for(int i=0; i < exceptionHandlers.length; i++) {
      ExceptionHandlerEntry exceptionHandler = exceptionHandlers[i];

      String[] r = exceptionHandler.resolve(constPool);

      out.println("    " + r[0]);
    }

    AttributeEntry[] attributes = attribute.getAttributes();

    out.println("  Attributes " + Converter.toHexString((short)attributes.length));

    for(int i=0; i < attributes.length; i++) {
      printEntry(attributes[i], out);
    }
  }

  public static void main(String args[]) throws Exception {
    ClassFile classFile = new ClassFile();
    classFile.read(args[0]);

    System.out.println("File length      :" + new File(args[0]).length());
    System.out.println("ClassFile length :" + classFile.length());

    classFile.write(args[1]);

    classFile.print(args[2]);
  }

}
