// ConstPool.java

package org.javalobby.classfile;

import java.io.*;
import java.util.*;

// implementation of pool behavior

/**
 * Class that represents a collection of entries for simulating constant pool.
 *
 * @version 1.0 02/22/2000
 * @author Alexander Shvets
 */
public final class ConstPool extends EntryCollection implements Pool {

  /* A package in which all const entries are defined */
//  private static String BASIC_PACKAGE = "org.javalobby.classfile";

  /* A table with predefined names for all constants that can exist
   * inside constant pool
   */
  private static Class[] table = new Class[13];
  static {
    table[0]                               = null;
    table[ConstPoolEntry.CONSTANT_UTF8]    = org.javalobby.classfile.Utf8Const.class;
    table[ConstPoolEntry.CONSTANT_INTEGER] = org.javalobby.classfile.IntegerConst.class;
    table[ConstPoolEntry.CONSTANT_FLOAT]   = org.javalobby.classfile.FloatConst.class;
    table[ConstPoolEntry.CONSTANT_LONG]    = org.javalobby.classfile.LongConst.class;
    table[ConstPoolEntry.CONSTANT_DOUBLE]  = org.javalobby.classfile.DoubleConst.class;
    table[ConstPoolEntry.CONSTANT_CLASS]   = org.javalobby.classfile.ClassConst.class;
    table[ConstPoolEntry.CONSTANT_STRING]  = org.javalobby.classfile.StringConst.class;
    table[ConstPoolEntry.CONSTANT_FIELD]   = org.javalobby.classfile.FieldConst.class;
    table[ConstPoolEntry.CONSTANT_METHOD]  = org.javalobby.classfile.MethodConst.class;
    table[ConstPoolEntry.CONSTANT_INTERFACEMETHOD] = org.javalobby.classfile.InterfaceMethodConst.class;
    table[ConstPoolEntry.CONSTANT_NAMEANDTYPE] = org.javalobby.classfile.NameAndTypeConst.class;
    table[ConstPoolEntry.CONSTANT_UNICODE]     = org.javalobby.classfile.UnicodeConst.class;
  }

  private short cnt;

  /**
   * Creates a collection of entries that all have basic parent - PoolEntry
   */
  public ConstPool() {
    super(PoolEntry.class, "Constant Pool");
  }

  /**
   * Get specified entry from a pool. Implementation of Pool interface.
   *
   * @param   index  the index of entry in a pool.
   * @return  specified entry
   */
  public PoolEntry getEntry(short index) {
    return (PoolEntry)entries.elementAt(index);
  }

  /**
   * Read const pool from input stream.
   *
   * @param  in  input stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void read(DataInput in) throws IOException {

    cnt = in.readShort();

    // First slot (0) is always unused
    entries.addElement(null);

    int i = 1;
    while(i < cnt) {
      PoolEntry entry = (PoolEntry)createEntry(in);
      entries.addElement(entry);

      // long and double constants occupy 2 entries of const pool
      if((entry instanceof LongConst) || (entry instanceof DoubleConst)) {
        entries.addElement(null);
        i++;
      }
      i++;
    }
  }

  /**
   * Creates a const entry for this constant pool.
   *
   * @param  in  input stream.
   * @return an entry with the specified type
   */
  protected Entry createEntry(DataInput in) throws IOException {
    Entry entry = null;

    int tag = in.readByte();

    try {
//      Class clazz = Class.forName(BASIC_PACKAGE + "." + table[tag]);
      Class clazz = table[tag];
      entry = (PoolEntry)clazz.newInstance();
    }
    catch(Exception e) {
      System.out.println("Unknown tag: " + tag);
    }

    if(entry != null)
      entry.read(in);

    return entry;
  }

  /**
   * Write constant pool to output stream.
   *
   * @param  out  output stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void write(DataOutput out) throws IOException {
    out.writeShort(cnt);

    int i = 1;
    while(i < cnt) {
      PoolEntry entry = (PoolEntry)entries.elementAt(i);
      entry.write(out);

      // long and double constants occupy 2 entries of const pool
      if((entry instanceof LongConst) || (entry instanceof DoubleConst))
        i++;
      i++;
    }
  }

  /**
   * Get number od entries in collection
   *
   * @return  number od entries in collection
   */
  public int size() {
    return cnt;
  }

  /**
   * Length of collection in bytes
   *
   * @return  length of collection in bytes
   */
  public long length() {
    int sz = entries.size();
    long len = 0;
    for(int i=1; i < sz; i++) {
      Entry e = (Entry)entries.elementAt(i);
      if(e != null)
        len += (long)e.length();
    }

    return len;
  }

  /**
   * Add pool entry to a constant pool.
   *
   * @param  pool entry  an entry to be added.
   */
  public short add(PoolEntry entry) {
    int index = entries.indexOf(entry);

    if(index == -1) {
      entries.addElement(entry);
      index = entries.size()-1;
    }

    cnt = (short)index;

    return cnt;
  }

  /**
   * Modify existing UTF value.
   *
   * @param  index the index of UTF8 constant to be modified.
   */
  public void modifyUTF(short index, String newValue) {
    PoolEntry entry = getEntry(index);
    if(entry instanceof Utf8Const) {
      entry = new Utf8Const(newValue);
      entries.setElementAt(entry, index);
    }
  }

}
