// EntryCollection.java

package org.javalobby.classfile;

import java.io.*;
import java.util.*;

// collection of entries for such essence as constant pool,
// lists of fields, methods, class attributes

/**
 * Class that represents a container, a collection of entries e.g.
 * constant pool, interfaces list, methods list etc.
 *
 * @version 1.0 02/22/2000
 * @author Alexander Shvets
 */
public class EntryCollection implements Entry {

  /** The vector that holds all entries in a collection */
  protected Vector entries = new Vector();

  /** The type of class that is allowable for elements of ths collection.
   *  Collection must contains only enities of specified type.
   */
  private Class clazz;

  private String type;

  /**
   * Creates a collection that can hold only elements of specified type.
   *
   * @param clazz  the type of allowed in this collection class
   */
  public EntryCollection(Class clazz, String type) {
    this.clazz = clazz;
    this.type  = type;
  }

  /**
   * Read this collection from input stream.
   *
   * @param  in  input stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void read(DataInput in) throws IOException {
    short cnt = in.readShort();

    for(int i=0; i < cnt; i++) {
      Entry entry = createEntry(in);
      entries.addElement(entry);
    }
  }

  /**
   * Creates an entry for this collection with the predefined type.
   *
   * @param  in  input stream.
   * @return an entry with the specified type
   */
  protected Entry createEntry(DataInput in) throws IOException {
    try {
      Entry entry = (Entry)clazz.newInstance();
      entry.read(in);
      return entry;
    }
    catch(Exception e) {
      throw new IOException(e.toString());
    }
  }

  /**
   * Write this collection to output stream.
   *
   * @param  out  output stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void write(DataOutput out) throws IOException {
    short cnt = (short)entries.size();
    out.writeShort(cnt);
    for(int i=0; i < cnt; i++)
      ((Entry)entries.elementAt(i)).write(out);
  }

  /**
   * Get type of this entry.
   *
   * @return  type of entry
   */
  public String getType() {
    return type;
  }

  /**
   * Length of collection in bytes
   *
   * @return  length of collection in bytes
   */
  public long length() {
    int sz = entries.size();
    int len = 0;
    for(int i=0; i < sz; i++) {
      len += (long)((Entry)entries.elementAt(i)).length();
    }

    return 2 + len;
  }

  /**
   * Get number od entries in collection
   *
   * @return  number od entries in collection
   */
  public int size() {
    return entries.size();
  }

  /**
   * Add an entry to a collection.
   *
   * @param  entry  an entry to be added.
   */
  public void add(Entry entry) {
    if(!entries.contains(entry))
      entries.addElement(entry);
  }

  /**
   * Remove an entry from a collection.
   *
   * @param  entry  an entry to be removed.
   */
  public void remove(Entry entry) {
    entries.removeElement(entry);
  }

  /**
   * Get entry that resides at the specified index.
   *
   * @param  index  a position of entry in this collection
   * @return an entry in this collection that resides at the specified index
   */
  public Entry elementAt(short index) {
    return (Entry)entries.elementAt(index);
  }

  /**
   * Set entry at the specified index in this collection.
   *
   * @param entry  an entry that should be set up
   * @param  index  a position in collection where new entry will be set up
   */
  public void setElementAt(Entry entry, short index) {
    entries.setElementAt(entry, index);
  }

  /**
   * Clone this collection.
   *
   * @return  a clone of this object
   */
  public Object clone() {
    EntryCollection e = new EntryCollection(clazz, type);
    e.entries = (Vector)entries.clone();
    return e;
  }

  /**
   * Returns a string representation of the object.
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    StringBuffer sb = new StringBuffer();

    int sz = entries.size();

    for(int i=0; i < sz; i++)
      sb.append(" " + entries.elementAt(i).toString());

    return "{(" + sz + ")" + sb + "}";
  }

}
