// FloatConst.java

package org.javalobby.classfile;

import java.io.*;

/**
 * Class that represents a "float" const entry.
 *
 * @version 1.0 02/22/2000
 * @author Alexander Shvets
 */
public class FloatConst implements ConstPoolEntry {

  /* the float value of this entry */
  private float value;

  /**
   * The default constructor that creates float constant.
   */
  public FloatConst() {}

  /**
   * Read this entry from input stream.
   *
   * @param  in  input stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void read(DataInput in) throws IOException {
    value = in.readFloat();
  }

  /**
   * Write this entry to output stream.
   *
   * @param  out  output stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void write(DataOutput out) throws IOException {
    out.writeByte(getTag());
    out.writeFloat(value);
  }

  /**
   * Get type of this entry.
   *
   * @return  type of entry
   */
  public String getType() {
    return "Float Const";
  }

  /**
   * Length of this entry in bytes (tag + size of float type).
   *
   * @return  length of entry
   */
  public long length() {
    return 1 + 4;
  }

  /**
   * Get the float value for this entry.
   *
   * @return  the float value for this entry
   */
  public float getValue() {
    return value;
  }

  /**
   * Get tag of this entry.
   *
   * @return  tag for entry
   */
  public byte getTag() {
    return CONSTANT_FLOAT;
  }

  /**
   * Resolve this entry.
   *
   * @param   pool  the pool
   * @return  the resolved information in form of an array
   */
  public String[] resolve(Pool constPool) {
    return new String[] { getType(), new Float(value).toString() };
  }

  /**
   * Compares two Objects for equality. Two float consts
   * will be equals if they both have the same value.
   *
   * @param   object  the reference object with which to compare.
   * @return  true if this object is the same as the obj
   *          argument; false otherwise.
   */
  public boolean equals(Object o) {
    if(o == null) return false;

    if(o instanceof FloatConst) {
      FloatConst c = (FloatConst)o;
      return (value == c.value);
    }

    return false;
  }

  /**
   * Returns a string representation of the object.
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "{" + getTag() + " " + value + "}";
  }

}
