// InterfaceEntry.java

package org.javalobby.classfile;

import java.io.*;

/**
 * Class that represents an interface entry.
 *
 * @version 1.0 02/22/2000
 * @author Alexander Shvets
 */
public class InterfaceEntry implements Entry, Resolvable {

  /* the index value that points to the name of this entry */
  private short nameIndex;

  /**
   * The default constructor that creates interface entry.
   */
  public InterfaceEntry() {}

  /**
   * Read this entry from input stream.
   *
   * @param  in  input stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void read(DataInput in) throws IOException {
    nameIndex = in.readShort();
  }

  /**
   * Write this entry to output stream.
   *
   * @param  out  output stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void write(DataOutput out) throws IOException {
    out.writeShort(nameIndex);
  }

  /**
   * Get type of this entry.
   *
   * @return  type of entry
   */
  public String getType() {
    return "Interface";
  }

  /**
   * Length of this entry in bytes (name index size).
   *
   * @return  length of entry
   */
  public long length() {
    return 2;
  }

  /**
   * Get the index that points to the name of this entry
   *
   * @return  the index that points to the name of this entry
   */
  public short getNameIndex() {
    return nameIndex;
  }

  /**
   * Resolve this entry.
   *
   * @param   constPool  the pool
   * @return  the resolved information in form of an array
   */
  public String[] resolve(Pool constPool) {
    ClassConst entry = (ClassConst)constPool.getEntry(nameIndex);
    short index = entry.getNameIndex();
    String name = ((Utf8Const)constPool.getEntry(index)).getValue();

    return new String[] { name.replace('/', '.') };
  }

  /**
   * Compares two Objects for equality. Two interface entries
   * will be equals if they both have the same value and name.
   *
   * @param   object  the reference object with which to compare.
   * @return  true if this object is the same as the obj
   *          argument; false otherwise.
   */
  public boolean equals(Object o) {
    if(o == null) return false;

    if(o instanceof InterfaceEntry) {
      InterfaceEntry c = (InterfaceEntry)o;

      return (nameIndex == c.nameIndex);
    }

    return false;
  }

  /**
   * Returns a string representation of the object.
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return Converter.toHexString(nameIndex);
  }

}
