// MemberEntry.java

package org.javalobby.classfile;

import java.io.*;
import java.util.*;

/**
 * Basic class that can represent either field or method entry.
 *
 * @version 1.0 02/22/2000
 * @author Alexander Shvets
 */
public abstract class MemberEntry implements Entry, Resolvable {

  /** access attributes for this member entry */
  protected AccessFlags accessFlags = new AccessFlags();

  /** the index value that points to the name of this entry */
  protected short nameIndex;


  /** the index value that points to value of this entry */
  protected short valueIndex;

  /** Attributes for this member entry */
  protected EntryCollection attributes =
                            new EntryCollection(AttributeEntry.class, "Attributes");

  /**
   * The default constructor that creates field or method entry.
   */
  public MemberEntry() {}

  /**
   * Read this entry from input stream.
   *
   * @param  in  input stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void read(DataInput in) throws IOException  {
    accessFlags.read(in);
    nameIndex   = in.readShort();
    valueIndex  = in.readShort();
    attributes.read(in);
  }

  /**
   * Write this entry to output stream.
   *
   * @param  out  output stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void write(DataOutput out) throws IOException {
    accessFlags.write(out);
    out.writeShort(nameIndex);
    out.writeShort(valueIndex);
    attributes.write(out);
  }

  /**
   * Length of this entry in bytes
   * (size of access flag + name index size + value index size +
   *  size of attributes).
   *
   * @return  length of entry
   */
  public long length() {
    return accessFlags.length() + 2 + 2 + attributes.length();
  }

  /**
   * Get access flags for this member entry.
   *
   * @return access flags for this member entry
   */
  public AccessFlags getAccessFlags() {
    return accessFlags;
  }

  /**
   * Set access flags for this member entry.
   *
   * @param flags  access flags for this member entry
   */
  public void setAccessFlags(short flags) {
    this.accessFlags.setAccessFlags(flags);
  }

  /**
   * Get the index that points to the name of this entry
   *
   * @return  the index that points to the name of this entry
   */
  public short getNameIndex() {
    return nameIndex;
  }

  /**
   * Get the index that points to the value of this entry
   *
   * @return  the index that points to the value of this entry
   */
  public short getValueIndex() {
    return valueIndex;
  }

  /**
   * Get the attributes for this member entry
   *
   * @return  the attributes for this member entry
   */
  public EntryCollection getAttributes() {
    return attributes;
  }

  /**
   * Get the name of this member entry
   *
   * @param constPool   the constant pool
   * @return  the name of this member entry
   */
  public String getName(Pool constPool) {
    Utf8Const entry = (Utf8Const)constPool.getEntry(nameIndex);

    return entry.getValue();
  }

  /**
   * Get the signature of this member entry
   *
   * @param constPool   the constant pool
   * @return  the signature of this member entry
   */
  public String getSignature(Pool constPool) {
    Utf8Const entry = (Utf8Const)constPool.getEntry(valueIndex);

    return entry.getValue();
  }

  /**
   * Returns a string representation of the object.
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "{" + accessFlags.getAccessFlags() + " " + nameIndex + " " +
           valueIndex + " " + attributes + "}";
  }

}
