// NameAndTypeConst.java

package org.javalobby.classfile;

import java.io.*;

/**
 * Class that represents a "name and type" const entry.
 *
 * @version 1.0 02/22/2000
 * @author Alexander Shvets
 */
public class NameAndTypeConst implements ConstPoolEntry {

  /* the index value that points to the name of this entry */
  private short nameIndex;

  /* the index value that points to value of this entry */
  private short valueIndex;

  /**
   * The default constructor that creates "name and type" constant.
   */
  public NameAndTypeConst() {}

  /**
   * Read this entry from input stream.
   *
   * @param  in  input stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void read(DataInput in) throws IOException {
    nameIndex = in.readShort();
    valueIndex = in.readShort();
  }

  /**
   * Write this entry to output stream.
   *
   * @param  out  output stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void write(DataOutput out) throws IOException {
    out.writeByte(getTag());
    out.writeShort(nameIndex);
    out.writeShort(valueIndex);
  }

  /**
   * Get type of this entry.
   *
   * @return  type of entry
   */
  public String getType() {
    return "Name And Type Const";
  }

  /**
   * Length of this entry in bytes.
   * (tag + name index size + value index size)
   *
   * @return  length of entry
   */
  public long length() {
    return 1 + 2 + 2;
  }

  /**
   * Get the index that points to the name of this entry
   *
   * @return  the index that points to the name of this entry
   */
  public short getNameIndex() {
    return nameIndex;
  }

  /**
   * Get the index that points to the value of this entry
   *
   * @return  the index that points to the value of this entry
   */
  public short getValueIndex() {
    return valueIndex;
  }

  /**
   * Get tag of this entry.
   *
   * @return  tag for entry
   */
  public byte getTag() {
    return CONSTANT_NAMEANDTYPE;
  }

  /**
   * Resolve this entry.
   *
   * @param   pool  the pool
   * @return  the resolved information in form of an array
   */
  public String[] resolve(Pool constPool) {
    Utf8Const entry1 = (Utf8Const)constPool.getEntry(nameIndex);
    Utf8Const entry2 = (Utf8Const)constPool.getEntry(valueIndex);

    return new String[] { getType(), entry1.getValue(), entry2.getValue() };
  }

  /**
   * Compares two Objects for equality. Two "name and type" consts
   * will be equals if they both have the same value and name.
   *
   * @param   object  the reference object with which to compare.
   * @return  true if this object is the same as the obj
   *          argument; false otherwise.
   */
  public boolean equals(Object o) {
    if(o == null) return false;

    if(o instanceof NameAndTypeConst) {
      NameAndTypeConst c = (NameAndTypeConst)o;
      return ((nameIndex == c.nameIndex) && (valueIndex == c.valueIndex));
    }

    return false;
  }

  /**
   * Returns a string representation of the object.
   *
   * @return  a string representation of the object.
   */
  public String toString() {
   return "{" + getTag() + " " + nameIndex + " " + valueIndex + "}";
  }

}
