// Signature.java

package org.javalobby.classfile;

import java.io.*;
import java.util.*;

/**
 * Class that represents a signature of a field.
 *
 * @version 1.0 02/22/2000
 * @author Alexander Shvets
 */
public class Signature {

  /* A map of correspondences for primitive types */
  private static Hashtable primitives = new Hashtable();

  static {
    primitives.put("V", "void");
    primitives.put("B", "byte");
    primitives.put("C", "char");
    primitives.put("D", "double");
    primitives.put("F", "float");
    primitives.put("I", "int");
    primitives.put("J", "long");
    primitives.put("S", "short");
    primitives.put("Z", "boolean");
  }

  /* A string representation of a signature for a field */
  private String key;

  /**
   * Constructs a signature with the specified key.
   *
   * @param   key   the string representation of a signature
   */
  public Signature(String key) {
    setKey(key);
  }

  /**
   * Get the key for this signature.
   *
   * @return  the key for this signature
   */
  public String getKey() {
    return key;
  }

  /**
   * Set the key for this signature.
   *
   * @param  key  the key for this signature
   */
  public void setKey(String key) {
    this.key = key;
  }

  /**
   * Determine if the signature is of primitive type.
   *
   * @return  true if the signature is of primitive type
   */
  public boolean isPrimitiveType() {
    if(key.length() == 1) {
      char ch = key.charAt(0);
      if(ch == 'B' || ch == 'C' || ch == 'D' || ch == 'F' ||
         ch == 'I' || ch == 'J' || ch == 'S' || ch == 'Z' || ch == 'V')
        return true;
    }

    return false;
  }

  /**
   * Determine if the signature is of array type.
   *
   * @return  true if the signature is of array type
   */
  public boolean isArrayType() {
    return key.startsWith("[");
  }

  /**
   * Determine if the signature is of reference type.
   *
   * @return  true if the signature is of reference type
   */
  public boolean isReferenceType() {
    return key.startsWith("L");
  }

  /**
   * Calculate the type inside signature.
   *
   * @return  the type that this signature contain
   */
  public String getTypeName() {
    return getTypeName0(key);
  }

  private static String getTypeName0(String key) {
    String typeName = (String)primitives.get(key);
    if(typeName != null) { // base types
      return typeName;
    }
    else if(key.startsWith("[")) { // array type
      typeName = getTypeName0(key.substring(1)) + "[]";
    }
    else if(key.startsWith("L")) { // object type
      return key.substring(1, key.length()-1);
    }

    return typeName;
  }

  /**
   * Calculate the type inside signature. Primitive types is ignored.
   *
   * @return  the type that this signature contain except of primitive types
   */
  public static String cleanName(String oldName) {
    int cnt = 0;
    while(oldName.charAt(cnt) == '[')
      cnt++;

    char ch = oldName.charAt(cnt);
    if(ch == 'B' || ch == 'C' || ch == 'D' || ch == 'F' ||
       ch == 'I' || ch == 'J' || ch == 'S' || ch == 'Z' ||
       ch == 'V') {
      return null;
    }
    else if(ch == 'L')
      return oldName.substring(cnt+1, oldName.length()-1);

    return oldName;
  }

  /**
   * Returns a string representation of the object.
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return getTypeName();
  }

}
