// StringConst.java

package org.javalobby.classfile;

import java.io.*;

/**
 * Class that represents a "string" const entry.
 *
 * @version 1.0 02/22/2000
 * @author Alexander Shvets
 */
public class StringConst implements ConstPoolEntry {

  /* the index value that points to name of this entry */
  private short nameIndex;

  /**
   * The default constructor that creates string constant.
   */
  public StringConst() {}

  /**
   * Read this entry from input stream.
   *
   * @param  in  input stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void read(DataInput in) throws IOException {
    nameIndex = in.readShort();
  }

  /**
   * Write this entry to output stream.
   *
   * @param  out  output stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void write(DataOutput out) throws IOException {
    out.writeByte(getTag());
    out.writeShort(nameIndex);
  }

  /**
   * Get type of this entry.
   *
   * @return  type of entry
   */
  public String getType() {
    return "String Const";
  }

  /**
   * Length of this entry in bytes.
   * (tag + name index size)
   *
   * @return  length of entry
   */
  public long length() {
    return 1 + 2;
  }

  /**
   * Get the index that points to the name of this entry
   *
   * @return  the index that points to the name of this entry
   */
  public short getNameIndex() {
    return nameIndex;
  }

  /**
   * Get tag of this entry.
   *
   * @return  tag for entry
   */
  public byte getTag() {
    return CONSTANT_STRING;
  }

  /**
   * Resolve this entry.
   *
   * @param   pool  the pool
   * @return  the resolved information in form of an array
   */
  public String[] resolve(Pool constPool) {
    String name = ((Utf8Const)constPool.getEntry(nameIndex)).getValue();
    return new String[] { getType(), name };
  }

  /**
   * Compares two Objects for equality. Two unicode consts
   * will be equals if they both have the same value-array.
   *
   * @param   object  the reference object with which to compare.
   * @return  true if this object is the same as the obj
   *          argument; false otherwise.
   */
  public boolean equals(Object o) {
    if(o == null) return false;

    if(o instanceof StringConst) {
      StringConst c = (StringConst)o;
      return (nameIndex == c.nameIndex);
    }

    return false;
  }

  /**
   * Returns a string representation of the object.
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "{" + getTag() + " " + nameIndex + "}";
  }

}
