// UnicodeConst.java

package org.javalobby.classfile;

import java.io.*;
import java.util.*;

/**
 * Class that represents an "unicode" const entry.
 *
 * @version 1.0 02/22/2000
 * @author Alexander Shvets
 */
public class UnicodeConst implements ConstPoolEntry {

  /* an array of bytes that holds unicode string */
  protected byte[] value;

  /**
   * A default constructor that creates unicode constant.
   */
  public UnicodeConst() {}

  /**
   * Read this entry from input stream.
   *
   * @param  in  input stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void read(DataInput in) throws IOException {
    short size = in.readShort();
    value      = new byte[size];
    in.readFully(value);
  }

  /**
   * Write this entry to output stream.
   *
   * @param  out  output stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void write(DataOutput out) throws IOException {
    short size = (short)value.length;
    out.writeShort(size);
    out.write(value, 0, size);
  }

  /**
   * Get type of this entry.
   *
   * @return  type of entry
   */
  public String getType() {
    return "Unicode Const";
  }

  /**
   * Length of this entry in bytes.
   * (tag + unicode length + unicode chars)
   *
   * @return  length of entry
   */
  public long length() {
    return 1 + 2 + value.length;
  }

  /**
   * Get the value-array for this entry.
   *
   * @return  the value-array for this entry
   */
  public byte[] getValue() {
    return value;
  }

  /**
   * Get tag of this entry.
   *
   * @return  tag for entry
   */
  public byte getTag() {
    return CONSTANT_UNICODE;
  }

  /**
   * Resolve this entry.
   *
   * @param   pool  the pool
   * @return  the resolved information in form of an array
   */
  public String[] resolve(Pool constPool) {
    return new String[] { getType(), new String(value) };
  }

  /**
   * Compares two Objects for equality. Two unicode consts
   * will be equals if they both have the same value-array.
   *
   * @param   object  the reference object with which to compare.
   * @return  true if this object is the same as the obj
   *          argument; false otherwise.
   */
  public boolean equals(Object o) {
    if(o == null) return false;

    if(o instanceof UnicodeConst) {
      UnicodeConst c = (UnicodeConst)o;
      return new String(value).equals(new String(c.value));
    }

    return false;
  }

  /**
   * Returns a string representation of the object.
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "{" + getTag() + " ...}";
  }

}
