// Utf8Const.java


package org.javalobby.classfile;

import java.io.*;

/**
 * Class that represents an "utf8" const entry.
 *
 * @version 1.0 02/22/2000
 * @author Alexander Shvets
 */
public class Utf8Const implements ConstPoolEntry {

  /* The utf8 string */
  private String value;

  /**
   * A default constructor that creates utf8 constant.
   */
  public Utf8Const() {}

  /**
   * A constructor that creates utf8 constant with the specified value.
   *
   * @param value  value that specifies utf8 string
   */
  public Utf8Const(String value) {
    this.value = value;
  }

  /**
   * Read this entry from input stream.
   *
   * @param  in  input stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void read(DataInput in) throws IOException {
    value = in.readUTF();
  }

  /**
   * Write this entry to output stream.
   *
   * @param  out  output stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void write(DataOutput out) throws IOException {
    out.writeByte(getTag());
    out.writeUTF((String)value);
  }

  /**
   * Get type of this entry.
   *
   * @return  type of entry
   */
  public String getType() {
    return "UTF8 Const";
  }

  /**
   * Length of this entry in bytes.
   * (tag + utf8 length + utf chars)
   *
   * @return  length of entry
   */
  public long length() {
    return 1 + 2 + value.length();
  }

  /**
   * Get the value-string for this entry.
   *
   * @return  the value-string for this entry
   */
  public String getValue() {
    return value;
  }

  /**
   * Get tag of this entry.
   *
   * @return  tag for entry
   */
  public byte getTag() {
    return CONSTANT_UTF8;
  }

  /**
   * Resolve this entry.
   *
   * @param   pool  the pool
   * @return  the resolved information in form of an array
   */
  public String[] resolve(Pool constPool) {
    return new String[] { getType(), value };
  }

  /**
   * Compares two Objects for equality. Two const pool entries
   * will be equals if they both have the same value.
   *
   * @param   object  the reference object with which to compare.
   * @return  true if this object is the same as the obj
   *          argument; false otherwise.
   */
  public boolean equals(Object o) {
    if(o == null) return false;

    if(o instanceof Utf8Const) {
      Utf8Const c = (Utf8Const)o;
      return value.equals(c.value);
    }

    return false;
  }

  /**
   * Returns a string representation of the object.
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return getType() + " " + value;
  }

}
