// CodeAttribute.java

package org.javalobby.classfile.attribute;

import java.io.*;
import java.util.*;

import org.javalobby.classfile.*;
import org.javalobby.classfile.instruction.*;

/**
 * An interpretator for "Code" attribute.
 *
 * @version 1.0 02/22/2000
 * @author Alexander Shvets
 */
public class CodeAttribute extends AttributeInterpretator {

  /* max stack value */
  private short maxStack;

  /* max locals value */
  private short maxLocals;

  private MethodBody methodBody;

  /* an array of exception handlers entries */
  private ExceptionHandlerEntry[] exceptionHandlers;

  /* an array of attributes for this code attribute */
  private AttributeEntry[] attributes;

  /**
   * Creates interpretator for "Code" attribute
   *
   * @param attribute  an original attribute
   */
  public CodeAttribute(AttributeEntry attribute) {
    super(attribute);
  }

  /**
   * Read this attribute from input stream.
   *
   * @param  in  input stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void read(DataInput in) throws IOException {
    maxStack  = in.readShort();
    maxLocals = in.readShort();

    int codeSz = in.readInt();

    methodBody = new MethodBody(codeSz);
    methodBody.read(in);

    short excSz = in.readShort();

    exceptionHandlers = new ExceptionHandlerEntry[excSz];
    for(int i=0; i < excSz; i++) {
      (exceptionHandlers[i] = new ExceptionHandlerEntry()).read(in);
    }

    short attrSz = in.readShort(); // attributes count

    attributes = new AttributeEntry[attrSz];
    for(int i=0; i < attrSz; i++) {
      (attributes[i] = new AttributeEntry()).read(in);
    }
  }

  /**
   * Write this attributre to output stream.
   *
   * @param  out  output stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void write(DataOutput out) throws IOException {
    out.writeShort(maxStack);
    out.writeShort(maxLocals);

    out.writeInt((int)methodBody.length());
    methodBody.write(out);

    short excSz = (short)exceptionHandlers.length;

    out.writeShort(excSz);

    for(int i=0; i < excSz; i++) {
      exceptionHandlers[i].write(out);
    }

    short attrSz = (short)attributes.length;

    out.writeShort(attrSz);

    for(int i=0; i < attrSz; i++) {
      attributes[i].write(out);
    }
  }

  /**
   * Get type of this entry.
   *
   * @return  type of entry
   */
  public String getType() {
    return "Code Attribute";
  }

  /**
   * Length of this attribute in bytes.
   *
   * @return  length
   */
  public long length() {
    short attrSz = (short)attributes.length;

    int attrLength = 0;
    for(int i=0; i < attrSz; i++) {
      attrLength += attributes[i].length();
    }

    return 2 + 4 +
           2 + 2 + 4 + methodBody.length()*1 +
           2 + exceptionHandlers.length*2*4 +
           2 + attrLength;
  }

  /**
   * Remove all attributes for this code attribute
   *
   * @return  true if there some changes in this attribute
   */
  public boolean clearCodeAttributes() {
    boolean changes = attributes.length > 0;

    if(changes) {
      attributes = new AttributeEntry[0];

      byte[] buffer = BytesProducer.getBytes(this);

      attribute.setBuffer(buffer);
    }

    return changes;
  }

  /**
   * Get max stack value
   *
   * @return  max stack value
   */
  public short getMaxStack() {
    return maxStack;
  }

  /**
   * Get max locals value
   *
   * @return  max locals value
   */
  public short getMaxLocals() {
    return maxLocals;
  }

  /**
   * Get the body for this code attribute
   *
   * @return  the body for this code attribute
   */
  public MethodBody getMethodBody() {
    return methodBody;
  }

  /**
   * Get an array of exception handler entries
   *
   * @return  an array of exception handler entries
   */
  public ExceptionHandlerEntry[] getExceptionHandlers() {
    return exceptionHandlers;
  }

  /**
   * Get an array of attributes
   *
   * @return  an array of attributes
   */
  public AttributeEntry[] getAttributes() {
    return attributes;
  }

}
