// ExceptionEntry.java

package org.javalobby.classfile.attribute;

import java.io.*;
import java.util.*;

import org.javalobby.classfile.*;

/**
 * Class that represents an exception entry in the exception table for
 * a method body.
 *
 * @version 1.0 02/22/2000
 * @author Alexander Shvets
 */
public class ExceptionEntry implements Entry, Resolvable {

  /* the index value that points to the name of this entry */
  private short nameIndex;

  /**
   * The default constructor that creates exception entry.
   */
  public ExceptionEntry() {}

  /**
   * Read this entry from input stream.
   *
   * @param  in  input stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void read(DataInput in) throws IOException {
    nameIndex = in.readShort();
  }

  /**
   * Write this entry to output stream.
   *
   * @param  out  output stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void write(DataOutput out) throws IOException {
    out.writeShort(nameIndex);
  }

  /**
   * Get type of this entry.
   *
   * @return  type of entry
   */
  public String getType() {
    return "Exception";
  }

  /**
   * Length of this entry in bytes (name index size).
   *
   * @return  length of entry
   */
  public long length() {
    return 2;
  }

  /**
   * Get the index that points to the name of this entry
   *
   * @return  the index that points to the name of this entry
   */
  public short getNameIndex() {
    return nameIndex;
  }

  /**
   * Resolve this entry.
   *
   * @param   constPool  the pool
   * @return  the resolved information in form of an array
   */
  public String[] resolve(Pool constPool) {
    PoolEntry entry = constPool.getEntry(nameIndex);
    String name = entry.resolve(constPool)[1];

    return new String[] { name };
  }

  /**
   * Compares two Objects for equality. Two exception entries
   * will be equals if they both have the same value and name.
   *
   * @param   object  the reference object with which to compare.
   * @return  true if this object is the same as the obj
   *          argument; false otherwise.
   */
  public boolean equals(Object o) {
    if(o == null) return false;

    if(o instanceof ExceptionEntry) {
      ExceptionEntry c = (ExceptionEntry)o;

      return (nameIndex == c.nameIndex);
    }

    return false;
  }

  /**
   * Returns a string representation of the object.
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[" + nameIndex + "]";
  }

}

