// ExceptionHandlerEntry.java


package org.javalobby.classfile.attribute;

import java.io.*;
import java.util.*;

import org.javalobby.classfile.*;

/**
 * Class that represents an exception handler entry.
 *
 * @version 1.0 02/22/2000
 * @author Alexander Shvets
 */
public class ExceptionHandlerEntry implements Entry, Resolvable {

  /* An index that points to start PC index */
  private short startPc;

  /* An index that points to end PC index */
  private short endPc;

  /* An index that points to handler PC index */
  private short handlerPc;

  /* An index that points to catch type index */
  private short catchType;

  /**
   * The default constructor that creates exception handler entry.
   */
  public ExceptionHandlerEntry() {}

  /**
   * Read this entry from input stream.
   *
   * @param  in  input stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void read(DataInput in) throws IOException {
    startPc   = in.readShort();
    endPc     = in.readShort();
    handlerPc = in.readShort();
    catchType = in.readShort();
  }

  /**
   * Write this entry to output stream.
   *
   * @param  out  output stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void write(DataOutput out) throws IOException {
    out.writeShort(startPc);
    out.writeShort(endPc);
    out.writeShort(handlerPc);
    out.writeShort(catchType);
  }

  /**
   * Get type of this entry.
   *
   * @return  type of entry
   */
  public String getType() {
    return "Exception Handler";
  }

  /**
   * Length of this entry in bytes
   * (start PC + end PC + handler PC + catch type index).
   *
   * @return  length of entry
   */
  public long length() {
    return 2 + 2 + 2 + 2;
  }

  /**
   * Get start PC index
   *
   * @return  start PC index
   */
  public short getStartPc() {
    return startPc;
  }

  /**
   * Get end PC index
   *
   * @return  end PC index
   */
  public short getEndPc() {
    return endPc;
  }

  /**
   * Get handler PC index
   *
   * @return  handler PC index
   */
  public short getHandlerPc() {
    return handlerPc;
  }

  /**
   * Get catch type index
   *
   * @return  catch type index
   */
  public short getCatchType() {
    return catchType;
  }

  /**
   * Resolve this entry.
   *
   * @param   pool  the pool
   * @return  the resolved information in form of an array
   */
  public String[] resolve(Pool constPool) {
    String name = "any";

    if(catchType != 0) {
      PoolEntry entry = constPool.getEntry(catchType);
      name = entry.resolve(constPool)[1].replace('/', '.');
    }

    String text = "(" + startPc + "-" + endPc + ", " + handlerPc + ") " + name;

    return new String[] { text };
  }

  /**
   * Returns a string representation of the object.
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[" + startPc + ", " + endPc + ", " + handlerPc + ", " +
           catchType + "]";
  }

}

