// InnerClassEntry.java

package org.javalobby.classfile.attribute;

import java.io.*;
import java.util.*;

import org.javalobby.classfile.*;

/**
 * Class that represents an inner class entry.
 *
 * @version 1.0 02/22/2000
 * @author Alexander Shvets
 */
public class InnerClassEntry implements Entry, Resolvable {

  /* an index that points to inner class in constant pool */
  private short innerClassIndex;

  /* an index that points to  outer class in constant pool */
  private short outerClassIndex;

  /* an index that points to  inner name of inner class in constant pool */
  private short innerNameIndex;

  /* Access flags for this inner class */
  private AccessFlags accessFlags = new AccessFlags();

  /**
   * The default constructor that creates inner class entry.
   */
  public InnerClassEntry() {}

  /**
   * Read this entry from input stream.
   *
   * @param  in  input stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void read(DataInput in) throws IOException {
    innerClassIndex = in.readShort();
    outerClassIndex = in.readShort();
    innerNameIndex  = in.readShort();

    accessFlags.read(in);
  }

  /**
   * Write this entry to output stream.
   *
   * @param  out  output stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void write(DataOutput out) throws IOException {
    out.writeShort(innerClassIndex);
    out.writeShort(outerClassIndex);
    out.writeShort(innerNameIndex);
    accessFlags.write(out);
  }

  /**
   * Get type of this entry.
   *
   * @return  type of entry
   */
  public String getType() {
    return "Inner Class";
  }

  /**
   * Length of this entry in bytes
   * (inner class + outer class + inner name + access flags).
   *
   * @return  length of entry
   */
  public long length() {
    return 2 + 2 + 2 + accessFlags.length();
  }

  /**
   * Get an index that points to a class const for inner class in constant pool
   *
   * @return  an inner class index
   */
  public short getInnerClassIndex() {
    return innerClassIndex;
  }

  /**
   * Get an index that points to a class const for outer class in constant pool
   *
   * @return  an outer class index
   */
  public short getOuterClassIndex() {
    return outerClassIndex;
  }

  /**
   * Get an index that points to a name for inner class in constant pool
   *
   * @return  an index
   */
  public short getInnerNameIndex() {
    return innerNameIndex;
  }

  /**
   * Get access flags for inner class entry.
   *
   * @return access flags
   */
  public AccessFlags getAccessFlags() {
    return accessFlags;
  }

  /**
   * Resolve this entry.
   *
   * @param   pool  the pool
   * @return  the resolved information in form of an array
   */
  public String[] resolve(Pool constPool) {
    PoolEntry ei1 = constPool.getEntry(innerClassIndex);
    String[] str1 = ei1.resolve(constPool);
    String innerClassFileName = str1[1].replace('/', '.');

    String outerClassName = "this";
    if(outerClassIndex != 0) {
      PoolEntry ei2 = constPool.getEntry(outerClassIndex);
      String[] str2 = ei2.resolve(constPool);
      outerClassName = str2[1].replace('/', '.');
    }

    String innerClassName = "this";
    if(innerNameIndex != 0) {
      PoolEntry ei3 = constPool.getEntry(innerNameIndex);
      String[] str3 = ei3.resolve(constPool);
      innerClassName = str3[1].replace('/', '.');
    }

    return new String[] {
      accessFlags.toString() + " " + outerClassName + "." + innerClassName +
      "(" + innerClassFileName + ")"
    };
  }

  /**
   * Returns a string representation of the object.
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[" + innerClassIndex + ", " + outerClassIndex + ", " +
            innerNameIndex + ", " + accessFlags.toString() + "]";
  }

}

