// LocalVariableEntry.java

package org.javalobby.classfile.attribute;

import java.io.*;
import java.util.*;

import org.javalobby.classfile.*;

/**
 * Class that represents a local variable entry.
 *
 * @version 1.0 02/22/2000
 * @author Alexander Shvets
 */
public class LocalVariableEntry implements Entry, Resolvable {

  /* An index that points to start PC index */
  private short startPc;

  /* An index that points to length */
  private short length;

  /* An index that points to name index */
  private short nameIndex;

  /* An index that points to description index */
  private short descrIndex;

  /* An index that points to index */
  private short index;

  /**
   * The default constructor that creates local variable entry.
   */
  public LocalVariableEntry() {}

  /**
   * Read this entry from input stream.
   *
   * @param  in  input stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void read(DataInput in) throws IOException {
    startPc    = in.readShort();
    length     = in.readShort();
    nameIndex  = in.readShort();
    descrIndex = in.readShort();
    index      = in.readShort();
  }

  /**
   * Write this entry to output stream.
   *
   * @param  out  output stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void write(DataOutput out) throws IOException {
    out.writeShort(startPc);
    out.writeShort(length);
    out.writeShort(nameIndex);
    out.writeShort(descrIndex);
    out.writeShort(index);
  }

  /**
   * Get type of this entry.
   *
   * @return  type of entry
   */
  public String getType() {
    return "Local Variable";
  }

  /**
   * Length of this entry in bytes
   * (start PC + length + name index + description index + index).
   *
   * @return  length of entry
   */
  public long length() {
    return 2 + 2 + 2 + 2 + 2;
  }

  /**
   * Get start PC index
   *
   * @return  start PC index
   */
  public short getStartPc() {
    return startPc;
  }

  /**
   * Get length value
   *
   * @return  length value
   */
  public short getLength() {
    return length;
  }

  /**
   * Get name index
   *
   * @return  name index
   */
  public short getNameIndex() {
    return nameIndex;
  }

  /**
   * Get description index
   *
   * @return  description index
   */
  public short getDescriptionIndex() {
    return descrIndex;
  }

  /**
   * Get index
   *
   * @return  index
   */
  public short getIndex() {
    return index;
  }

  /**
   * Resolve this entry.
   *
   * @param   pool  the pool
   * @return  the resolved information in form of an array
   */
  public String[] resolve(Pool constPool) {
    PoolEntry ei1 = constPool.getEntry(nameIndex);
    String[] str1 = ei1.resolve(constPool);

    PoolEntry ei2 = constPool.getEntry(descrIndex);
    String[] str2 = ei2.resolve(constPool);

    Signature signature = new Signature(str2[1]);

    String text = "(" + startPc + "-" + (startPc + length) + ", " +
                  index + ") " + signature + " " + str1[1];

    return new String[] { text };
  }

  /**
   * Returns a string representation of the object.
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "(" + startPc + ", " + length + ", " + nameIndex + ", " +
            descrIndex + ", " + index + ")";
  }

}
