// MethodBody.java

package org.javalobby.classfile.attribute;

import java.io.*;
import java.util.*;

import org.javalobby.classfile.*;
import org.javalobby.classfile.instruction.*;

/**
 * Class that the body of method, i.e. a sequence of JVM instructions.
 *
 * @version 1.0 02/22/2000
 * @author Alexander Shvets
 */
public final class MethodBody implements Entry {

  /* check id method's body has some changes */
  private boolean isBodyChanged = false;

  /* list of instructions thor this method */
  private Vector instructions = new Vector();

  /* length of method body in bytes */
  private int length;

  /*
   * Creates a method body with specified space for instructions
   *
   * @param length the length of method's body in bytes
   */
  public MethodBody(int length) {
    this.length = length;
  }

  /**
   * Read this entry from input stream.
   *
   * @param  in  input stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void read(DataInput in) throws IOException {
    int ic = 0;

    while(ic < length) {
      boolean wide = false;
      byte tag = in.readByte();
      ic++;

      if((tag & 0xff) == Opcode.WIDE) { // Read next tag after wide byte
        wide = true;
        tag  = in.readByte();
        ic++;
      }

      if((tag & 0xff) == Opcode.TABLESWITCH ||
         (tag & 0xff) == Opcode.LOOKUPSWITCH) {
        for(;(ic % 4) != 0; ic++) {
          in.readByte(); // skip padding
        }
      }

      Instruction instruction = InstructionFactory.create(tag, wide);

      instruction.read(in);

      instructions.addElement(instruction);

      ic += instruction.length()-1;
    }
  }

  /**
   * Write this entry to output stream.
   *
   * @param  out  output stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void write(DataOutput out) throws IOException {
    for(int i=0, ic=0; i < instructions.size(); i++) {
      Instruction instruction = (Instruction)instructions.elementAt(i);

      byte tag     = instruction.getTag();
      boolean wide = instruction.isWide();

      if(wide) {
        out.writeByte(Opcode.WIDE);
        ic++;
      }

      out.writeByte(tag);
      ic++;

      if((tag & 0xff) == Opcode.TABLESWITCH ||
         (tag & 0xff) == Opcode.LOOKUPSWITCH) {
        for(;(ic % 4) != 0; ic++) {
          out.writeByte((byte)0);
        }
      }

      instruction.write(out);
      ic += instruction.length()-1;
    }
  }

  /**
   * Get type of this entry.
   *
   * @return  type of entry
   */
  public String getType() {
    return "Method Body";
  }

  /**
   * Length of this entry in bytes
   *
   * @return  length of entry.
   */
  public long length() {
    return length;
  }

  /**
   * Get an instruction from a body.
   *
   * @param index  the index of instruction in the list of instructions
   * @return an instruction that occupies the specified index
   */
  public Instruction getInstruction(int index) {
    return (Instruction)instructions.elementAt(index);
  }

  /**
   * Set an instruction to a body at the specified index.
   *
   * @param an instruction that should be setted up at the specified index
   * @param index  the index
   */
  public void setInstruction(Instruction instruction, int index) {
    instructions.setElementAt(instruction, index);

    calculateLength();
    isBodyChanged = true;
  }

  /**
   * Insert (add) an instruction after the specified index.
   *
   * @param an instruction that will be added
   * @param index  the index
   */
  public void insertInstruction(Instruction instruction, int index) {
    instructions.insertElementAt(instruction, index);

    calculateLength();
    isBodyChanged = true;
  }

  /**
   * Remove an instruction from instructions list.
   *
   * @param an instruction that will be removed
   */
  public void removeInstruction(Instruction instruction) {
    instructions.removeElement(instruction);

    calculateLength();
    isBodyChanged = true;
  }

  /**
   * Check if body of method hase some changes.
   *
   * @return  true if body of method hase some changes; false - otherwise
   */
  public boolean isBodyChanged() {
    return isBodyChanged;
  }

  /**
   * Get list of instructions for this body.
   *
   * @return  list of instructions
   */
  public Instruction[] getInstructions() {
    Instruction[] array = new Instruction[instructions.size()];

    instructions.copyInto(array);

    return array;
  }

  /**
   * The number of instructions in instructions list
   *
   * @return  the number of instructions in instructions list
   */
  public int size() {
    return instructions.size();
  }

  /* Calculate length for each instruction in instructions list after any
   * change in this list.
   */
  private void calculateLength() {
    int sz = 0;
    for(int i=0; i < instructions.size(); i++) {
      Instruction instruction = (Instruction)instructions.elementAt(i);

      byte tag     = instruction.getTag();
      boolean wide = instruction.isWide();

      if(wide) {
        sz++;
      }

      sz++;

      if((tag & 0xff) == Opcode.TABLESWITCH ||
         (tag & 0xff) == Opcode.LOOKUPSWITCH) {
        for(;(sz % 4) != 0; sz++);
      }

      sz += instruction.length()-1;
    }

    length = sz;
  }

}
