// LookupSwitchInstruction.java

package org.javalobby.classfile.instruction;

import java.io.*;

import org.javalobby.classfile.*;

/**
 * This class is an implementation of Instruction interface for LOOKUPSWITCH
 * instruction.
 *
 * @version 1.0 02/22/2000
 * @author Alexander Shvets
 */
public class LookupSwitchInstruction extends DefaultInstruction {

  /* default offset value */
  private int defaultOffset;

  /* default npairs value */
  private int npairs;

  /**
   * This constructor that creates LOOKUPSWITCH instruction with a specified
   * tag value. Flag "wide" for this instruction is meaningless.
   *
   * @param tag  the tag value
   */
  public LookupSwitchInstruction(byte tag) {
    super(tag, false);
  }

  /**
   * Read this instruction from input stream.
   *
   * @param  in  input stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void read(DataInput in) throws IOException {
    defaultOffset = in.readInt();
    npairs        = in.readInt();

    int restLength = npairs*(4 + 4);

    buffer = new byte[restLength];

    in.readFully(buffer);
  }

  /**
   * Write this instruction to output stream.
   *
   * @param  out  output stream.
   * @exception  IOException if an I/O error occurs.
   */
  public void write(DataOutput out) throws IOException {
    out.writeInt(defaultOffset);
    out.writeInt(npairs);

    out.write(buffer, 0, buffer.length);
  }

  /**
   * Get the length of instruction in bytes
   * (tag + default offset + npairs + size of buffer-parameter)
   *
   * @return  the length of instruction in bytes
   */
  public long length() {
    return (long)(1 + 4 + 4 + buffer.length);
  }

  /**
   * Get the default offset
   *
   * @return the default offset
   */
  public int getDefaultOffset() {
    return defaultOffset;
  }

  /**
   * Get the npairs value
   *
   * @return the npairs value
   */
  public int getNpairs() {
    return npairs;
  }

  /**
   * Resolve this entry.
   *
   * @param   pool  the pool
   * @return  the resolved information in form of an array
   */
  public String[] resolve(Pool constPool) {
    StringBuffer sb = new StringBuffer(" ");
    for(int i=0; i < buffer.length/4; i+=2) {
      sb.append("<");
      sb.append(Integer.toHexString(Converter.getInt(buffer, i)));
      sb.append(" ");
      sb.append(Integer.toHexString(Converter.getInt(buffer, i+1)));
      sb.append(">");
      sb.append(" ");
    }

    return new String[] {
      getMnemonic(),
      "default: " + Integer.toHexString(defaultOffset) + "; " +
      "num: "     + Integer.toHexString(npairs) + "; " +
      "{" + sb.toString() + "}"
    };
  }

  /**
   * Compares two Objects for equality. Two lookup switch instructions
   * will be equals if they both have the same tag value, wide flag,
   * parameter, default offset and npairs values.
   *
   * @param   object  the reference object with which to compare.
   * @return  true if this object is the same as the obj
   *          argument; false otherwise.
   */
  public boolean equals(Object object) {
    if(object instanceof LookupSwitchInstruction) {
      LookupSwitchInstruction instruction = (LookupSwitchInstruction)object;
      if(super.equals(instruction) &&
         this.defaultOffset == instruction.defaultOffset &&
         this.npairs == instruction.npairs)
        return true;
    }

    return false;
  }

}

