// OpcodeTable.java

package org.javalobby.classfile.instruction;

/**
 * This class represents a table for descriptions of all JVM instructions.
 *
 * @version 1.0 02/22/2000
 * @author Alexander Shvets
 */
public class OpcodeTable implements Opcode {

  /* an opcode table */
  private static OpcodeDescription[] table = new OpcodeDescription[256];

  /* Disable creation of any instance of this class - it is only a container
   * for static methods.
   */
  private OpcodeTable() {}

  /**
   * Get a description of specified instruction
   *
   * @param  opcode the opcode of instruction
   * @return  an instruction that corresponds to a given opcode
   */
  public static OpcodeDescription getDescription(byte opcode) {
    return table[opcode & 0xff];
  }

  /**
   * Get an opcode of specified instruction
   *
   * @param  mnemonic the mnemonic of instruction
   * @return  the opcode of instruction
   */
  public static int getOpcode(String mnemonic) {
    int opcode = -1;
    for(int i=0; i < table.length; i++) {
      OpcodeDescription description = table[i];
      if(description.mnemonic.equals(mnemonic)) {
        opcode = i;
        break;
      }
    }

    return opcode;
  }

  /**
   * Get the opcode table.
   *
   * @return  Get the opcode table
   */
  public static OpcodeDescription[] getTable() {
    return table;
  }

  /* This list contains instructions that support wide mode.
   * wide instruction is placed in front of an instruction that
   * accesses a local variable.
   */
  private static int[] wideSupport = new int[] {
    ILOAD, LLOAD, FLOAD, DLOAD, ALOAD,
    ISTORE, LSTORE, FSTORE, DSTORE, ASTORE,
    RET,
    IINC
  };

  /* this list contains conditional instructions */
   private static int[] conditions = new int[] {
    IFEQ, IFNE, IFLT, IFGE, IFGT, IFLE,
    IF_ICMPEQ, IF_ICMPNE, IF_ICMPLT, IF_ICMPGE, IF_ICMPGT, IF_ICMPLE,
    IF_ACMPEQ, IF_ACMPNE,
    GOTO, JSR, RET,
    IFNULL, IFNONNULL, GOTO_W, JSR_W,
  };

  /* simple seek in the list */
  private static boolean seek(int opcode, int[] list) {
    for(int i=0; i < list.length; i++) {
      if(list[i] == opcode)
        return true;
    }

    return false;
  }

  /**
   * Check if specified instruction support wide mode.
   *
   * @param tag the tag for instruction
   * @return  true if instruction supports wide mode; false otherwise.
   */
  public static boolean isSupportWideMode(byte tag) {
    return seek((tag & 0xff), wideSupport);
  }

  /**
   * Check if specified instruction belongs to conditions.
   *
   * @param tag the tag for instruction
   * @return  true if instruction  belongs to conditions; false otherwise.
   */
  public static boolean isCondition(byte tag) {
    return seek((tag & 0xff), conditions);
  }

/* initialization of opcode table */
static {
  table[NOP]         = new OpcodeDescription("nop", 1);
  table[ACONST_NULL] = new OpcodeDescription("aconst_null", 1);

  table[ICONST_M1]   = new OpcodeDescription("iconst_m1", 1);

  table[ICONST_0]    = new OpcodeDescription("iconst_0", 1);
  table[ICONST_1]    = new OpcodeDescription("iconst_1", 1);
  table[ICONST_2]    = new OpcodeDescription("iconst_2", 1);
  table[ICONST_3]    = new OpcodeDescription("iconst_3", 1);
  table[ICONST_4]    = new OpcodeDescription("iconst_4", 1);
  table[ICONST_5]    = new OpcodeDescription("iconst_5", 1);

  table[LCONST_0]    = new OpcodeDescription("lconst_0", 1);
  table[LCONST_1]    = new OpcodeDescription("lconst_1", 1);

  table[FCONST_0]    = new OpcodeDescription("fconst_0", 1);
  table[FCONST_1]    = new OpcodeDescription("fconst_1", 1);
  table[FCONST_2]    = new OpcodeDescription("fconst_2", 1);

  table[DCONST_0]    = new OpcodeDescription("dconst_0", 1);
  table[DCONST_1]    = new OpcodeDescription("dconst_1", 1);

  table[BIPUSH]      = new OpcodeDescription("bipush", 2);
  table[SIPUSH]      = new OpcodeDescription("sipush", 3);

  table[LDC]         = new RefOpcodeDescription("ldc", 2);
  table[LDC_W]       = new RefOpcodeDescription("ldc_w", 3);
  table[LDC2_W]      = new RefOpcodeDescription("ldc2_w", 3);

  table[ILOAD]       = new OpcodeDescription("iload", 2);
  table[LLOAD]       = new OpcodeDescription("lload", 2);
  table[FLOAD]       = new OpcodeDescription("fload", 2);
  table[DLOAD]       = new OpcodeDescription("dload", 2);
  table[ALOAD]       = new OpcodeDescription("aload", 2);

  table[ILOAD_0]     = new OpcodeDescription("iload_0", 1);
  table[ILOAD_1]     = new OpcodeDescription("iload_1", 1);
  table[ILOAD_2]     = new OpcodeDescription("iload_2", 1);
  table[ILOAD_3]     = new OpcodeDescription("iload_3", 1);

  table[LLOAD_0]     = new OpcodeDescription("lload_0", 1);
  table[LLOAD_1]     = new OpcodeDescription("lload_1", 1);
  table[LLOAD_2]     = new OpcodeDescription("lload_2", 1);
  table[LLOAD_3]     = new OpcodeDescription("lload_3", 1);

  table[FLOAD_0]     = new OpcodeDescription("fload_0", 1);
  table[FLOAD_1]     = new OpcodeDescription("fload_1", 1);
  table[FLOAD_2]     = new OpcodeDescription("fload_2", 1);
  table[FLOAD_3]     = new OpcodeDescription("fload_3", 1);

  table[DLOAD_0]     = new OpcodeDescription("dload_0", 1);
  table[DLOAD_1]     = new OpcodeDescription("dload_1", 1);
  table[DLOAD_2]     = new OpcodeDescription("dload_2", 1);
  table[DLOAD_3]     = new OpcodeDescription("dload_3", 1);

  table[ALOAD_0]     = new OpcodeDescription("aload_0", 1);
  table[ALOAD_1]     = new OpcodeDescription("aload_1", 1);
  table[ALOAD_2]     = new OpcodeDescription("aload_2", 1);
  table[ALOAD_3]     = new OpcodeDescription("aload_3", 1);

  table[IALOAD]      = new OpcodeDescription("iaload", 1);
  table[LALOAD]      = new OpcodeDescription("laload", 1);
  table[FALOAD]      = new OpcodeDescription("faload", 1);
  table[DALOAD]      = new OpcodeDescription("daload", 1);
  table[AALOAD]      = new OpcodeDescription("aaload", 1);
  table[BALOAD]      = new OpcodeDescription("baload", 1);
  table[CALOAD]      = new OpcodeDescription("caload", 1);
  table[SALOAD]      = new OpcodeDescription("saload", 1);

  table[ISTORE]      = new OpcodeDescription("istore", 2);
  table[LSTORE]      = new OpcodeDescription("lstore", 2);
  table[FSTORE]      = new OpcodeDescription("fstore", 2);
  table[DSTORE]      = new OpcodeDescription("dstore", 2);
  table[ASTORE]      = new OpcodeDescription("astore", 2);

  table[ISTORE_0]    = new OpcodeDescription("istore_0", 1);
  table[ISTORE_1]    = new OpcodeDescription("istore_1", 1);
  table[ISTORE_2]    = new OpcodeDescription("istore_2", 1);
  table[ISTORE_3]    = new OpcodeDescription("istore_3", 1);

  table[LSTORE_0]    = new OpcodeDescription("lstore_0", 1);
  table[LSTORE_1]    = new OpcodeDescription("lstore_1", 1);
  table[LSTORE_2]    = new OpcodeDescription("lstore_2", 1);
  table[LSTORE_3]    = new OpcodeDescription("lstore_3", 1);

  table[FSTORE_0]    = new OpcodeDescription("fstore_0", 1);
  table[FSTORE_1]    = new OpcodeDescription("fstore_1", 1);
  table[FSTORE_2]    = new OpcodeDescription("fstore_2", 1);
  table[FSTORE_3]    = new OpcodeDescription("fstore_3", 1);

  table[DSTORE_0]    = new OpcodeDescription("dstore_0", 1);
  table[DSTORE_1]    = new OpcodeDescription("dstore_1", 1);
  table[DSTORE_2]    = new OpcodeDescription("dstore_2", 1);
  table[DSTORE_3]    = new OpcodeDescription("dstore_3", 1);

  table[ASTORE_0]    = new OpcodeDescription("astore_0", 1);
  table[ASTORE_1]    = new OpcodeDescription("astore_1", 1);
  table[ASTORE_2]    = new OpcodeDescription("astore_2", 1);
  table[ASTORE_3]    = new OpcodeDescription("astore_3", 1);

  table[IASTORE]     = new OpcodeDescription("iastore", 1);
  table[LASTORE]     = new OpcodeDescription("lastore", 1);
  table[FASTORE]     = new OpcodeDescription("fastore", 1);
  table[DASTORE]     = new OpcodeDescription("dastore", 1);
  table[AASTORE]     = new OpcodeDescription("aastore", 1);
  table[BASTORE]     = new OpcodeDescription("bastore", 1);
  table[CASTORE]     = new OpcodeDescription("castore", 1);
  table[SASTORE]     = new OpcodeDescription("sastore", 1);

  table[POP]         = new OpcodeDescription("pop", 1);
  table[POP2]        = new OpcodeDescription("pop2", 1);

  table[DUP]         = new OpcodeDescription("dup", 1);
  table[DUP_X1]      = new OpcodeDescription("dup_x1", 1);
  table[DUP_X2]      = new OpcodeDescription("dup_x2", 1);
  table[DUP2]        = new OpcodeDescription("dup2", 1);
  table[DUP2_X1]     = new OpcodeDescription("dup2_x1", 1);
  table[DUP2_X2]     = new OpcodeDescription("dup2_x2", 1);
  table[SWAP]        = new OpcodeDescription("swap", 1);

  table[IADD]        = new OpcodeDescription("iadd", 1);
  table[LADD]        = new OpcodeDescription("ladd", 1);
  table[FADD]        = new OpcodeDescription("fadd", 1);
  table[DADD]        = new OpcodeDescription("dadd", 1);

  table[ISUB]        = new OpcodeDescription("isub", 1);
  table[LSUB]        = new OpcodeDescription("lsub", 1);
  table[FSUB]        = new OpcodeDescription("fsub", 1);
  table[DSUB]        = new OpcodeDescription("dsub", 1);

  table[IMUL]        = new OpcodeDescription("imul", 1);
  table[LMUL]        = new OpcodeDescription("lmul", 1);
  table[FMUL]        = new OpcodeDescription("fmul", 1);
  table[DMUL]        = new OpcodeDescription("dmul", 1);

  table[IDIV]        = new OpcodeDescription("idiv", 1);
  table[LDIV]        = new OpcodeDescription("ldiv", 1);
  table[FDIV]        = new OpcodeDescription("fdiv", 1);
  table[DDIV]        = new OpcodeDescription("ddiv", 1);

  table[IREM]        = new OpcodeDescription("irem", 1);
  table[LREM]        = new OpcodeDescription("lrem", 1);
  table[FREM]        = new OpcodeDescription("frem", 1);
  table[DREM]        = new OpcodeDescription("drem", 1);

  table[INEG]        = new OpcodeDescription("ineg", 1);
  table[LNEG]        = new OpcodeDescription("lneg", 1);
  table[FNEG]        = new OpcodeDescription("fneg", 1);
  table[DNEG]        = new OpcodeDescription("dneg", 1);

  table[ISHL]        = new OpcodeDescription("ishl", 1);
  table[LSHL]        = new OpcodeDescription("lshl", 1);

  table[ISHR]        = new OpcodeDescription("ishr", 1);
  table[LSHR]        = new OpcodeDescription("lshr", 1);

  table[IUSHR]       = new OpcodeDescription("iushr", 1);
  table[LUSHR]       = new OpcodeDescription("lushr", 1);

  table[IAND]        = new OpcodeDescription("iand", 1);
  table[LAND]        = new OpcodeDescription("land", 1);

  table[IOR]         = new OpcodeDescription("ior", 1);
  table[LOR]         = new OpcodeDescription("lor", 1);

  table[IXOR]        = new OpcodeDescription("ixor", 1);
  table[LXOR]        = new OpcodeDescription("lxor", 1);

  table[IINC]        = new OpcodeDescription("iinc", 3);

  table[I2L]         = new OpcodeDescription("i2l", 1);
  table[I2F]         = new OpcodeDescription("i2f", 1);
  table[I2D]         = new OpcodeDescription("i2d", 1);

  table[L2I]         = new OpcodeDescription("l2i", 1);
  table[L2F]         = new OpcodeDescription("l2f", 1);
  table[L2D]         = new OpcodeDescription("l2d", 1);

  table[F2I]         = new OpcodeDescription("f2i", 1);
  table[F2L]         = new OpcodeDescription("f2l", 1);
  table[F2D]         = new OpcodeDescription("f2d", 1);

  table[D2I]         = new OpcodeDescription("d2i", 1);
  table[D2L]         = new OpcodeDescription("d2l", 1);
  table[D2F]         = new OpcodeDescription("d2f", 1);

  table[I2B]         = new OpcodeDescription("i2b", 1);
  table[I2C]         = new OpcodeDescription("i2c", 1);
  table[I2S]         = new OpcodeDescription("i2s", 1);

  table[LCMP]        = new OpcodeDescription("lcmp", 1);
  table[FCMPL]       = new OpcodeDescription("fcmpl", 1);
  table[FCMPG]       = new OpcodeDescription("fcmpg", 1);
  table[DCMPL]       = new OpcodeDescription("dcmpl", 1);
  table[DCMPG]       = new OpcodeDescription("dcmpg", 1);

  table[IFEQ]        = new OpcodeDescription("ifeq", 3);
  table[IFNE]        = new OpcodeDescription("ifne", 3);
  table[IFLT]        = new OpcodeDescription("iflt", 3);
  table[IFGE]        = new OpcodeDescription("ifge", 3);
  table[IFGT]        = new OpcodeDescription("ifgt", 3);
  table[IFLE]        = new OpcodeDescription("ifle", 3);

  table[IF_ICMPEQ]   = new OpcodeDescription("if_icmpeq", 3);
  table[IF_ICMPNE]   = new OpcodeDescription("if_icmpne", 3);
  table[IF_ICMPLT]   = new OpcodeDescription("if_icmplt", 3);
  table[IF_ICMPGE]   = new OpcodeDescription("if_icmpge", 3);
  table[IF_ICMPGT]   = new OpcodeDescription("if_icmpgt", 3);
  table[IF_ICMPLE]   = new OpcodeDescription("if_icmple", 3);

  table[IF_ACMPEQ]   = new OpcodeDescription("if_acmpeq", 3);
  table[IF_ACMPNE]   = new OpcodeDescription("if_acmpne", 3);

  table[GOTO]        = new OpcodeDescription("goto", 3);
  table[JSR]         = new OpcodeDescription("jsr", 3);
  table[RET]         = new OpcodeDescription("ret", 2);
  table[TABLESWITCH] = new OpcodeDescription("tableswitch", -1); // depends on alignment
  table[LOOKUPSWITCH]= new OpcodeDescription("lookupswitch", -1); // depends on alignment

  table[IRETURN]     = new OpcodeDescription("ireturn", 1);
  table[LRETURN]     = new OpcodeDescription("lreturn", 1);
  table[FRETURN]     = new OpcodeDescription("freturn", 1);
  table[DRETURN]     = new OpcodeDescription("dreturn", 1);
  table[ARETURN]     = new OpcodeDescription("areturn", 1);
  table[RETURN]      = new OpcodeDescription("return", 1);

  table[GETSTATIC]   = new RefOpcodeDescription("getstatic", 3);
  table[PUTSTATIC]   = new RefOpcodeDescription("putstatic", 3);
  table[GETFIELD]    = new RefOpcodeDescription("getfield", 3);
  table[PUTFIELD]    = new RefOpcodeDescription("putfield", 3);

  table[INVOKEVIRTUAL]    = new RefOpcodeDescription("invokevirtual", 3);
  table[INVOKESPECIAL]    = new RefOpcodeDescription("invokespecial", 3);
  table[INVOKESTATIC]     = new RefOpcodeDescription("invokestatic", 3);
  table[INVOKEINTERFACE]  = new RefOpcodeDescription("invokeinterface", 5);
  table[XXXUNUSEDXXX]     = new OpcodeDescription("***unused***", 1);

  table[NEW]              = new RefOpcodeDescription("new", 3);
  table[NEWARRAY]         = new RefOpcodeDescription("newarray", 2);
  table[ANEWARRAY]        = new RefOpcodeDescription("anewarray", 3);
  table[ARRAYLENGTH]      = new OpcodeDescription("arraylength", 1);

  table[ATHROW]           = new OpcodeDescription("athrow", 1);
  table[CHECKCAST]        = new RefOpcodeDescription("checkcast", 3);
  table[INSTANCEOF]       = new RefOpcodeDescription("instanceof", 3);
  table[MONITORENTER]     = new OpcodeDescription("monitorenter", 1);
  table[MONITOREXIT]      = new OpcodeDescription("monitorexit", 1);

  table[WIDE]             = new OpcodeDescription("wide", 1);
  table[MULTIANEWARRAY]   = new RefOpcodeDescription("multianewarray", 4);
  table[IFNULL]           = new OpcodeDescription("ifnull", 3);
  table[IFNONNULL]        = new OpcodeDescription("ifnonnull", 3);
  table[GOTO_W]           = new OpcodeDescription("goto_w", 5);
  table[JSR_W]            = new OpcodeDescription("jsr_w", 5);

  //quick opcodes:
  table[LDC_QUICK]                 = new RefOpcodeDescription("ldc_quick", 2);
  table[LDC_W_QUICK]               = new RefOpcodeDescription("ldc_w_quick", 3);
  table[LDC2_W_QUICK]              = new RefOpcodeDescription("ldc2_w_quick", 3);
  table[GETFIELD_QUICK]            = new RefOpcodeDescription("getfield_quick", 3);
  table[PUTFIELD_QUICK]            = new RefOpcodeDescription("putfield_quick", 3);
  table[GETFIELD2_QUICK]           = new RefOpcodeDescription("getfield2_quick", 3);
  table[PUTFIELD2_QUICK]           = new RefOpcodeDescription("putfield2_quick", 3);
  table[GETSTATIC_QUICK]           = new RefOpcodeDescription("getstatic_quick", 3);
  table[PUTSTATIC_QUICK]           = new RefOpcodeDescription("putstatic_quick", 3);
  table[GETSTATIC2_QUICK]          = new RefOpcodeDescription("getstatic2_quick", 3);
  table[PUTSTATIC2_QUICK]          = new RefOpcodeDescription("putstatic2_quick", 3);

  table[INVOKEVIRTUAL_QUICK]       = new RefOpcodeDescription("invokevirtual_quick", 3);
  table[INVOKENONVIRTUAL_QUICK]    = new RefOpcodeDescription("invokenonvirtual_quick", 3);
  table[INVOKESUPER_QUICK]         = new RefOpcodeDescription("invokesuper_quick", 3);
  table[INVOKESTATIC_QUICK]        = new RefOpcodeDescription("invokestatic_quick", 3);
  table[INVOKEINTERFACE_QUICK]     = new RefOpcodeDescription("invokeinterface_quick", 5);
  table[INVOKEVIRTUALOBJECT_QUICK] = new RefOpcodeDescription("invokevirtualobject_quick", 3);

  table[0xdc]                      = new OpcodeDescription("***unused***", 1);

  table[NEW_QUICK]                 = new RefOpcodeDescription("new_quick", 3);
  table[ANEWARRAY_QUICK]           = new RefOpcodeDescription("anewarray_quick", 3);
  table[MULTIANEWARRAY_QUICK]      = new RefOpcodeDescription("multianewarray_quick", 4);

  table[CHECKCAST_QUICK]           = new RefOpcodeDescription("checkcast_quick", 3);
  table[INSTANCEOF_QUICK]          = new RefOpcodeDescription("instanceof_quick", 3);
  table[INVOKEVIRTUAL_QUICK_W]     = new RefOpcodeDescription("invokevirtual_quick_w", 3);
  table[GETFIELD_QUICK_W]          = new RefOpcodeDescription("getfield_quick_w", 3);
  table[PUTFIELD_QUICK_W]          = new RefOpcodeDescription("putfield_quick_w", 3);

  // Unused opcodes:
  table[0xe5]                      = new OpcodeDescription("***unused***", 1);
  table[0xe6]                      = new OpcodeDescription("***unused***", 1);
  table[0xe7]                      = new OpcodeDescription("***unused***", 1);
  table[0xe8]                      = new OpcodeDescription("***unused***", 1);
  table[0xe9]                      = new OpcodeDescription("***unused***", 1);
  table[0xea]                      = new OpcodeDescription("***unused***", 1);
  table[0xeb]                      = new OpcodeDescription("***unused***", 1);
  table[0xec]                      = new OpcodeDescription("***unused***", 1);
  table[0xed]                      = new OpcodeDescription("***unused***", 1);
  table[0xee]                      = new OpcodeDescription("***unused***", 1);
  table[0xef]                      = new OpcodeDescription("***unused***", 1);
  table[0xf0]                      = new OpcodeDescription("***unused***", 1);
  table[0xf1]                      = new OpcodeDescription("***unused***", 1);
  table[0xf2]                      = new OpcodeDescription("***unused***", 1);
  table[0xf3]                      = new OpcodeDescription("***unused***", 1);
  table[0xf4]                      = new OpcodeDescription("***unused***", 1);
  table[0xf5]                      = new OpcodeDescription("***unused***", 1);
  table[0xf6]                      = new OpcodeDescription("***unused***", 1);
  table[0xf7]                      = new OpcodeDescription("***unused***", 1);
  table[0xf8]                      = new OpcodeDescription("***unused***", 1);
  table[0xf9]                      = new OpcodeDescription("***unused***", 1);
  table[0xfa]                      = new OpcodeDescription("***unused***", 1);
  table[0xfb]                      = new OpcodeDescription("***unused***", 1);
  table[0xfc]                      = new OpcodeDescription("***unused***", 1);
  table[0xfd]                      = new OpcodeDescription("***unused***", 1);

  // Reserved opcodes:
  table[BREAKPOINT]                = new OpcodeDescription("breakpoint", 1);
  table[IMPDEP1]                   = new OpcodeDescription("impdep1", 1);
  table[IMPDEP2]                   = new OpcodeDescription("impdep2", 1);

}

}

