// SerFile.java

package org.javalobby.serfile;

import java.io.*;
import java.util.*;

import org.javalobby.classfile.*;

/*
 * Class that represents serialized file
 *
 * stream: STREAM_MAGIC STREAM_VERSION contents
 *
 * @version 1.0 02/22/2000
 * @author Alexander Shvets
 */
public class SerFile implements SerConstants {
  /** Table that contains all handles to allocated elements of serialized file */
  private HandlesTable handlesTable = new HandlesTable();

  /** tracks changes in serialized file */
  private boolean changed = false;

  /** stream magic value */
  protected short streamMagic;

  /** stream version value */
  protected short streamVersion;

  /** contents of serialized file */
  protected Contents contents;

  /**
   * Length of serialized file in bytes.
   *
   * @return  length of serialized file in bytes
   */
  public long length() {
    return 2 + 2 + contents.length(); // stream magic, stream version, contents length
  }

  public void read(String serName) throws IOException {
    FileInputStream fis = new FileInputStream(serName);
    read(fis);
  }

  public void read(InputStream is) throws IOException {
    DataInputStream in = new DataInputStream(new BufferedInputStream(is, 8192));

    streamMagic = in.readShort();

    if(streamMagic != STREAM_MAGIC) {
      throw new StreamCorruptedException("Illegal stream magic number '0x" +
                             Converter.toHexString(streamMagic) + "'");
    }

    streamVersion = in.readShort();

    if(streamVersion != STREAM_VERSION) {
      throw new StreamCorruptedException("Illegal stream version '0x" +
                      Converter.toHexString(streamVersion) + "'" + " expected '0x" +
                      Converter.toHexString(STREAM_VERSION) + "'" );
    }

    contents = new Contents(handlesTable);

    contents.read(in);

    changed = false;

    in.close();
  }

  public void write(String fileName) throws IOException {
    write(new FileOutputStream(fileName));
  }

  public void write(OutputStream os) throws IOException {
    DataOutputStream out = new DataOutputStream(new BufferedOutputStream(os, 8192));

    out.writeShort(streamMagic);
    out.writeShort(streamVersion);

    contents.write(out);

    changed = false;

    out.close();
  }

  public boolean isChanged() {
    return changed;
  }

  public void setChanged(boolean changed) {
    this.changed = changed;
  }

  public short getStreamMagic() {
    return streamMagic;
  }

  public short getStreamVersion() {
    return streamVersion;
  }

  public Contents getContents() {
    return contents;
  }

  public HandlesTable getHandlesTable() {
    return handlesTable;
  }

  public void print(String fileName) throws IOException {
    PrintWriter out = new PrintWriter(new FileWriter(fileName));

    out.println();
    out.println("Stream magic  : 0x" + Converter.toHexString(streamMagic, ""));
    out.println("Stream version: 0x" + Converter.toHexString(streamVersion, ""));

    out.println("\nContents:");

    for(short i=0; i < contents.size(); i++) {
      ContentEntry contentEntry = (ContentEntry)contents.elementAt(i);
      Entry value = contentEntry.getValue();

      if(value instanceof HandleSerEntry) {
        HandleSerEntry entry = (HandleSerEntry)value;
        out.println(entry.getType() + " (" + entry.getHandle() + ")");
      }
      else {
        out.println(contentEntry);
      }
    }

    out.print("\nReference table:");

    Enumeration e = handlesTable.elements();

    if(!e.hasMoreElements()) {
      out.print("empty");
    }
    else {
      out.println();

      while(e.hasMoreElements()) {
        out.println(e.nextElement());
      }
    }
  }

  public static void main(String args[]) {
    SerFile serFile = new SerFile();

    try {
      serFile.read(args[0]);

      System.out.println("serfile's length : " + serFile.length());

      serFile.print(args[2]);

      if(args.length > 1) {
        serFile.write(args[1]);
      }
    }
    catch(Exception e) {
      e.printStackTrace();
      System.out.println(e);

      try {
        serFile.print(args[2]);
      }
      catch(Exception e2) {}
    }
  }

}

