// ClassSeeker.java

//package org.javalobby.task;

import java.io.*;

import org.javalobby.classfile.*;
import org.javalobby.task.basic.*;
import org.javalobby.task.event.*;
import org.javalobby.task.stripper.*;
import org.javalobby.task.*;
import org.javalobby.classfile.instruction.*;
import org.javalobby.classfile.attribute.*;


class SeekTask extends Task {

  private ConstPool constPool;

  private String zipFileName;

  private PrintWriter logWriter;

  public SeekTask(PrintWriter logWriter) {
    super(null);

    this.logWriter = logWriter;
  }

  public void body(ClassFile classFile) {
    logWriter.println("Class: " + classFile.getClassName().replace('/', '.'));

    constPool = classFile.getConstPool();

    // fields
    logWriter.println("Fields:");
    EntryCollection fields = classFile.fields();
    for(short i=0; i < fields.size(); i++) {
      FieldEntry entry = (FieldEntry)fields.elementAt(i);
      String[] str = entry.resolve(constPool);

      logWriter.println("  " + Converter.merge(str, " "));
    }

    // methods
    logWriter.println("Methods:");
    EntryCollection methods = classFile.methods();
    for(short i=0; i < methods.size(); i++) {
      MethodEntry entry = (MethodEntry)methods.elementAt(i);
      String[] str = entry.resolve(constPool);

      logWriter.println("  " + Converter.merge(str, " "));

      analyzeMethod(entry);
    }

  }

  private void analyzeMethod(MethodEntry methodEntry) {
    AttributeEntry attribute =
                   methodEntry.getAttribute(AttributeEntry.CODE, constPool);

    if(attribute != null) {
      CodeAttribute codeAttribute = new CodeAttribute(attribute);

      analyzeMethodBody(codeAttribute.getMethodBody().getInstructions());
    }
  }

  private void analyzeMethodBody(Instruction[] instructions) {
    for(int i=0; i < instructions.length; i++) {
      Instruction instruction = instructions[i];

      byte tag = instruction.getTag();
      OpcodeDescription description = OpcodeTable.getDescription(tag);

      if(description.length > 1) {
        RankTable rankTable = ParameterRanking.getInstance();
        String group = rankTable.getRank(tag).key;

        if(group.equals(ParameterRanking.METHOD_RANK) ||
           group.equals(ParameterRanking.IMETHOD_RANK)) {
          String str = instruction.resolve(constPool)[1];

          logWriter.println("    Method invocation: " + str);
        }
      }
    }
  }

  public void action() throws IOException {
    logWriter.println();
  }

  public void closeZip() {
    super.closeZip();
    logWriter.println("Archive's end.\n");
    zipFileName = null;
  }

}

// An example for batch processing of Strip-task
public class ClassSeeker extends GroupTask
                         implements NotificationListener {

  private PrintWriter logWriter;

  public ClassSeeker(Task task, String from, boolean recursive, PrintWriter logWriter) {
    super(task, from, recursive);

    this.logWriter = logWriter;

    this.addNotificationListener(this);
    task.addNotificationListener(this);
  }

  public void notification(NotificationEvent event) {
    String classFileName = event.getNotification();

    if(classFileName.endsWith(".class")) {
      logWriter.println("File : " + classFileName);
    }
    else if(classFileName.endsWith(".jar") || classFileName.endsWith(".zip")) {
      logWriter.println("Archive : " + classFileName + "\n");
    }
  }

  public static void main(String[] argv) {
    String dir = null;
    String log = null;
    boolean recursively = true;
    try {
      dir = new File(argv[0]).getPath();
      log = new File(argv[1]).getPath();
    }
    catch(Exception e) {
      System.out.println("\nUsage: java ClassSeeker dir log [recursively]");
      System.out.println("Where dir is a directory or archive;");
      System.out.println("      recursively is boolean\n");
      System.out.println("Author : " + Constants.AUTHOR);
      System.out.println("e-mail : " + Constants.EMAIL);
      return;
    }

    try {
      recursively = Boolean.valueOf(argv[2]).booleanValue();
    }
    catch(Exception e) {}

    try {
      PrintWriter logWriter = new PrintWriter(new FileWriter(log));

      SeekTask task = new SeekTask(logWriter);
  
      GroupTask groupTask = new ClassSeeker(task, dir, recursively, logWriter);

      System.out.println("Seek group command started...\n");

      groupTask.processing();

      System.out.println("Seek group command finished...\n");

      logWriter.close();

      System.out.println(groupTask.getProcessedFiles() + " files processed.");
      System.out.println(groupTask.getReplacements() + " replacements occured.");
    }
    catch(Exception e) {
      System.out.println(e);
    }
  }

}
