// GroupTask.java

package org.javalobby.task.basic;

import java.io.*;
import java.util.*;
import java.util.zip.*;

import org.javalobby.task.event.*;

// This class define basic behavior for processing some task
// for group of files (batch processing).
// It acts as bean-generator of events SelectionEvent

public class GroupTask {

  private NotificationSupport support = new NotificationSupport(this);

  private GroupSupport groupSupport = new GroupSupport(this);

  private int processedFiles, replacements, movements;

  private Task task;
  private String fromDirName;
  private String[] filesList;
  private boolean recursively;

  public GroupTask(Task task, String fromDirName, boolean recursively) {
    this.task        = task;
    this.fromDirName = fromDirName;
    this.recursively = recursively;

    replacements = 0;
    movements    = 0;
  }

  public GroupTask(Task task, String fromDirName, String[] filesList) {
    this.task        = task;
    this.fromDirName = fromDirName;
    this.filesList   = filesList;

    replacements = 0;
    movements    = 0;
  }

  public GroupTask(Task task, String[] filesList) {
    this(task, ".", filesList);
  }

  public int getProcessedFiles() {
    return processedFiles;
  }

  public int getReplacements() {
    return replacements;
  }

  public int getMovements() {
    return movements;
  }

  public void clearReplacements() {
    replacements = 0;
  }

  public void processing() throws IOException {
    processedFiles = 0;

    if(filesList != null) {
      for(int i=0; i < filesList.length; i++) {
        processingFile(fromDirName + File.separator + filesList[i]);
      }
    }
    else  {
      processingDir(fromDirName, true);
    }
  }

  public void processing(String[] filesList) {
    for(int i=0; i < filesList.length; i++) {
      String name = fromDirName + File.separator + filesList[i];
      try {
        processingFile(name);
      }
      catch (FileNotFoundException e1) {
        fireNotification("", "File " + name + " does not found.");
        continue;
      }
      catch (IOException e2) {
        fireNotification("", e2 + " when analyzing " + name + ".");
//        e2.printStackTrace();
        continue;
      }
    }
  }

  private void processingDir(String fromFileName, boolean first)
                             throws IOException {
    File fromFile = new File(fromFileName);

    if(!fromFile.exists())
      throw new IOException("File <" + fromFileName + "> doesn't exist.\n");

    if(fromFile.isFile()) {
      processingFile(fromFileName);
    }
    else {
      if(first || recursively) {
        String[] files = fromFile.list();
        for(int i=0; i < files.length; i++)
          processingDir(fromFileName + File.separator + files[i], false);
      }
    }
  }

  private String getExtension(String name) {
    String ext = null;
    int pos = name.lastIndexOf('.');
    if(pos > 0 && pos < name.length()-1)
      ext = name.substring(pos+1).toLowerCase();

    return ext;
  }

  private void processingFile(String fromFileName) throws IOException {
    File fromFile = new File(fromFileName);
    if(!fromFile.exists())
      throw new IOException("File <" + fromFileName + "> doesn't exist.");

    fireNewFileSelected(fromFileName);

    String ext = getExtension(fromFileName);
    if(ext != null) {
      if(ext.equals("class")) {
        fireNotification(fromFileName, "Analyzing class file");
        task.solve(fromFileName);

        replacements += task.getReplacements();
        if(task.isMoved())
          ++movements;
        processedFiles++;

        return;
      }
      else if(ext.equals("zip") || ext.equals("jar")) {
        fireNotification(fromFileName, "Analyzing archive");
        processingArchive(fromFileName);

        return;
      }
    }

    fireNotification(fromFileName, "Copying file");
    task.copy(fromDirName, fromFileName);
    if(task.isMoved())
      ++movements;

    processedFiles++;
  }

  private String getDirName(String dirName1, String dirName2) {
    if(dirName1.length() == 0)
      return (dirName2.length() == 0) ? "" : dirName2 + File.separator;
    else
      return (dirName2.length() == 0) ? dirName1 + File.separator :
              dirName1 + File.separator + dirName2 + File.separator;
  }

  private void processingArchive(String fromFileName) throws IOException {
    try {
      ZipFile zipFile = new ZipFile(fromFileName);
      Enumeration e = zipFile.entries();
      while(e.hasMoreElements()) {
        ZipEntry zipEntry = (ZipEntry)e.nextElement();
        String entryName  = zipEntry.getName();
        fireNotification(entryName, "Analyzing archive entry");

        String ext = getExtension(entryName);
        if(ext != null) {
          if(ext.equalsIgnoreCase("class")) {
            task.solve(zipFile, zipEntry);

            replacements += task.getReplacements();
            if(task.isMoved())
              ++movements;
            processedFiles++;

            continue;
          }
        }

        task.copy(zipFile, zipEntry);
        if(task.isMoved())
          ++movements;
        processedFiles++;
      }
    }
    finally {
      task.closeZip();
    }
  }

  public synchronized void addNotificationListener(NotificationListener l) {
    support.addNotificationListener(l);
  }
  public synchronized void removeNotificationListener(NotificationListener l) {
    support.removeNotificationListener(l);
  }

  // fire NotificationEvent to registered listeners
  protected void fireNotification(String notification, String description) {
    support.fireNotification(notification, description);
  }


  public synchronized void addGroupListener(GroupListener l) {
    groupSupport.addGroupListener(l);
  }
  public synchronized void removeGroupListener(GroupListener l) {
    groupSupport.removeGroupListener(l);
  }

  // fire NotificationEvent to registered listeners
  protected void fireNewFileSelected(String fileName) {
    groupSupport.fireNewFileSelected(fileName);
  }


}
