// CompactTask.java

package org.javalobby.task.compactor;

import java.io.*;
import java.util.*;

import org.javalobby.classfile.*;
import org.javalobby.classfile.*;
import org.javalobby.classfile.attribute.*;
import org.javalobby.classfile.instruction.*;
import org.javalobby.task.basic.*;

public class CompactTask extends Task {

  private ConstPool constPool;

  private Vector classesList, discoveredClassesList, dllList;
  private PackagesBounds bounds;

  public CompactTask(String to, Vector classesList,
                    Vector discoveredClassesList, Vector dllList,
                    PackagesBounds bounds) {
    super(to);

    this.classesList           = classesList;
    this.discoveredClassesList = discoveredClassesList;
    this.dllList               = dllList;
    this.bounds                = bounds;
  }

  public void body(ClassFile classFile) {
//    System.out.println("classFile.getClassName() " + classFile.getClassName());
    constPool = classFile.getConstPool();

    // linear analyze of constant pool
    for(short i=1; i < constPool.size(); i++) {
      PoolEntry constant = (PoolEntry)constPool.elementAt(i);
      if(constant == null) continue;

      if(constant instanceof ClassConst) {
        String name = constant.resolve(constPool)[1];
        if(name.startsWith("["))
          tryAdd(Signature.cleanName(name));
        else
          tryAdd(name);
      }
    }

    // we can miss analyze for super class, interfaces and class attributes:
    // InnerClasses, because they all are used tag CONSTANT_CLASS, and
    // therefore they are recognized on linear analyze stage

    // fields

    EntryCollection fields = classFile.fields();
    for(short i=0; i < fields.size(); i++) {
      FieldEntry field = (FieldEntry)fields.elementAt(i);
      short valueIndex = field.getValueIndex();
      Utf8Const entry = (Utf8Const)constPool.getEntry(valueIndex);

      tryAdd(Signature.cleanName(entry.getValue()));
    }

    // methods

    EntryCollection methods = classFile.methods();
    for(short i=0; i < methods.size(); i++) {
      MethodEntry method = (MethodEntry)methods.elementAt(i);

      short valueIndex = method.getValueIndex();
      Utf8Const entry = (Utf8Const)constPool.getEntry(valueIndex);

      MethodSignature methodSignature = new MethodSignature(entry.getValue());
      Signature[] parameters = methodSignature.getParameters();

      for(int j=0; j < parameters.length; j++) {
        tryAdd(Signature.cleanName(parameters[j].getKey()));
      }

      try {
        analyzeMethod(method);
//        System.out.println("method " + method.getName(constPool));
      }
      catch(Exception e) {
        e.printStackTrace();
      }
    }
  }

  private void analyzeMethod(MethodEntry methodEntry) throws Exception {
    AttributeEntry attribute =
                   methodEntry.getAttribute(AttributeEntry.CODE, constPool);

    if(attribute == null)
      return;

    CodeAttribute codeAttribute = new CodeAttribute(attribute);

    MethodBody methodBody = codeAttribute.getMethodBody();

    analyzeMethodBody(methodBody.getInstructions());

    ExceptionHandlerEntry[] exceptionHandlers =
                            codeAttribute.getExceptionHandlers();

    for(int i=0; i < exceptionHandlers.length; i++) {
      ExceptionHandlerEntry exceptionHandler = exceptionHandlers[i];
      short catchType = exceptionHandler.getCatchType();
      if(catchType == 0)
        continue;

      PoolEntry ei = constPool.getEntry(catchType);
      String name = ei.resolve(constPool)[1];

      tryAdd(name);
    }
  }

  //  1. find "invokestatic" and get its metod
  //  2. if method name starts with "class$" or "forName"
  //    2.1. get previous instruction "ldc"
  //    2.2. resolve this string
  private void analyzeMethodBody(Instruction[] instructions) {
    for(int i=0; i < instructions.length; i++) {
      Instruction instruction = instructions[i];

      byte tag = instruction.getTag();
      OpcodeDescription description = OpcodeTable.getDescription(tag);

      if(description.length > 1) {
        RankTable rankTable = ParameterRanking.getInstance();
        String group      = rankTable.getRank(tag).key;

        if(group.equals(ParameterRanking.METHOD_RANK) ||
           group.equals(ParameterRanking.IMETHOD_RANK)) {
          String str = instruction.resolve(constPool)[1];
          StringTokenizer st = new StringTokenizer(str);
          if(st.hasMoreTokens()) {
            st.nextToken();  // "Method" or "Interface Method"
            st.nextToken(); //signature

            String methodName = st.nextToken();
            if(methodName.equals("forName") || methodName.startsWith("class$")) {
              String s = getLDCValue(instructions, i);
              if(s != null)
                tryAdd(s);
            }
            else if(methodName.equals("loadLibrary")) {
              String s = getLDCValue(instructions, i);
              if(s != null && !dllList.contains(s))
                dllList.addElement(s);
            }
          }
        }
      }
    }
  }

  private String getLDCValue(Instruction[] instructions, int i) {
    for(int cnt = i-1; cnt >= 0; cnt--) {
      Instruction instruction = instructions[cnt];

      byte tag = instruction.getTag();
      OpcodeDescription description = OpcodeTable.getDescription(tag);

      if(description.length > 1) {
        RankTable rankTable = ParameterRanking.getInstance();
        String group      = rankTable.getRank(tag).key;

        if(group.equals(ParameterRanking.LDC_RANK)) {
          String str = instruction.resolve(constPool)[1];
//          System.out.println("str " + str);
          StringTokenizer st = new StringTokenizer(str);
          st.nextToken();
          if(st.hasMoreTokens()) {
            String name = st.nextToken();
            if(name != null) {
              try {
                Class.forName(name);
              }
              catch(Throwable ex) {
                name = null;
              }
            }
            return name;
          }
        }
      }
    }

    return null;
  }

  private void tryAdd(String className) {
    if(className == null) return;

    className = className.replace('/', '.');

    if((bounds == null) || (!bounds.contains(className)))
      if(!classesList.contains(className) &&
         !discoveredClassesList.contains(className))
        discoveredClassesList.addElement(className);
  }

}
