// CollectInfoTask.java

package org.javalobby.task.obfuscator;

import java.io.*;
import java.util.*;

import org.javalobby.classfile.*;
import org.javalobby.classfile.*;
import org.javalobby.classfile.attribute.*;
import org.javalobby.task.basic.*;
import org.javalobby.task.util.*;

public class CollectInfoTask extends Task {

  Vector fieldsList        = new Vector();
  Vector methodsList       = new Vector();
  Vector otherFieldsList   = new Vector();
  Vector otherMethodsList  = new Vector();
  Vector stringsList       = new Vector();
  Vector innerNamesList    = new Vector();
  Vector nativeMethodsList = new Vector();

  private String qualifiedClassName;
  private ConstPool constPool;

  private NamesTable names;

  public CollectInfoTask(NamesTable names) {
    super(null);

    this.names = names;
  }

  public void body(final ClassFile classFile) {
    constPool          = classFile.getConstPool();
    qualifiedClassName = classFile.getClassName();

    fieldsList.removeAllElements();
    methodsList.removeAllElements();
    otherFieldsList.removeAllElements();
    otherMethodsList.removeAllElements();
    stringsList.removeAllElements();
    innerNamesList.removeAllElements();

    // 1. Collect class names that belong to obfuscation area
    names.addClassName(qualifiedClassName);

    // 2. Find names, that we can meet inside constant pool for current class
    //    (not only field and method names of this class - fields and methods
    //    of current class + fields and methods of other classes, that
    //    it uses inside own const pool). If class uses name from other class,
    //    we add this one in form of utilize dependency (other, current)

    for(short i=1; i < constPool.size(); i++) {
      PoolEntry e = (PoolEntry)constPool.elementAt(i);
      if(e == null) continue;

      if(e instanceof FieldConst) {
        analyzeField((FieldConst)constPool.elementAt(i));
      }
      else if(e instanceof MethodConst) {
        analyzeMethod((MethodConst)constPool.elementAt(i));
      }
      else if(e instanceof StringConst) {
        analyzeString((StringConst)constPool.elementAt(i));
      }
    }

    EntryCollection fields = classFile.fields();
    for(short i=0; i < fields.size(); i++)
      analyzeField((FieldEntry)fields.elementAt(i));

    EntryCollection methods = classFile.methods();
    for(short i=0; i < methods.size(); i++)
      analyzeMethod((MethodEntry)methods.elementAt(i));

    EntryCollection attributes = classFile.attributes();
    for(short i=0; i < attributes.size(); i++) {
      AttributeEntry attribute = (AttributeEntry)attributes.elementAt(i);
      String attrName = attribute.getName(constPool);
      if(attrName.equals(AttributeEntry.INNER_CLASSES))
        analyzeInnerClassesAttribute(new InnerClassesAttribute(attribute));
    }

    addNames(fieldsList);
    addNames(methodsList);
    addNames(otherFieldsList);
    addNames(otherMethodsList);

    addInnerNames(innerNamesList);
  }

  private void addNames(Vector list) {
    for(int i=0; i < list.size(); i++) {
      String name = (String)list.elementAt(i);
      if(stringsList.contains(name))
        names.addName(qualifiedClassName, name, true);
      else if(nativeMethodsList.contains(name))
        names.addName(qualifiedClassName, name, true);
      else
        names.addName(qualifiedClassName, name, false);
    }
  }

  private void addInnerNames(Vector list) {
    for(int i=0; i < list.size(); i++) {
      String name = (String)list.elementAt(i);
      if(stringsList.contains(name))
        names.addInnerName(qualifiedClassName, name, true);
      else if(nativeMethodsList.contains(name))
        names.addName(qualifiedClassName, name, true);
      else
        names.addInnerName(qualifiedClassName, name, false);
    }
  }

  private void analyzeField(FieldConst e) {
    short nameIndex = e.getNameIndex();
    short valueIndex = e.getValueIndex();

    ClassConst e1       = (ClassConst)constPool.getEntry(nameIndex);
    NameAndTypeConst e2 = (NameAndTypeConst)constPool.getEntry(valueIndex);

    short nameIndex1  = e1.getNameIndex();
    short nameIndex2  = e2.getNameIndex();

    Utf8Const utf0   = (Utf8Const)constPool.getEntry(nameIndex1);
    Utf8Const utf1   = (Utf8Const)constPool.getEntry(nameIndex2);

    String className = utf0.getValue();
    String fieldName = utf1.getValue();

    if(className.equals(qualifiedClassName)) {
      if(!fieldsList.contains(fieldName))
        fieldsList.addElement(fieldName);
    }
    else {
      if(!otherFieldsList.contains(fieldName))
        otherFieldsList.addElement(fieldName);
    }

    names.addDependency(className, qualifiedClassName, fieldName);
  }

  private void analyzeMethod(MethodConst e) {
    short nameIndex = e.getNameIndex();
    short valueIndex = e.getValueIndex();

    ClassConst e1       = (ClassConst)constPool.getEntry(nameIndex);
    NameAndTypeConst e2 = (NameAndTypeConst)constPool.getEntry(valueIndex);

    short nameIndex1  = e1.getNameIndex();
    short nameIndex2  = e2.getNameIndex();

    Utf8Const utf0    = (Utf8Const)constPool.getEntry(nameIndex1);
    Utf8Const utf1    = (Utf8Const)constPool.getEntry(nameIndex2);

    String className  = utf0.getValue();
    String methodName = utf1.getValue();

    if(className.equals(qualifiedClassName)) {
      if(!methodsList.contains(methodName))
        methodsList.addElement(methodName);
    }
    else {
      if(!otherMethodsList.contains(methodName))
        otherMethodsList.addElement(methodName);
    }

    names.addDependency(className, qualifiedClassName, methodName);
  }

  private void analyzeString(StringConst e) {
    short nameIndex = e.getNameIndex();

    Utf8Const utf = (Utf8Const)constPool.getEntry(nameIndex);
    String string = utf.getValue();

    if(!stringsList.contains(string))
      stringsList.addElement(string);
  }

  private void analyzeField(FieldEntry e) {
    short nameIndex  = e.getNameIndex();

    Utf8Const e1     = (Utf8Const)constPool.getEntry(nameIndex);
    String fieldName = e1.getValue();

    if(!fieldsList.contains(fieldName))
      fieldsList.addElement(fieldName);
  }

  private void analyzeMethod(MethodEntry methodEntry) {
    short nameIndex  = methodEntry.getNameIndex();

    Utf8Const e1      = (Utf8Const)constPool.getEntry(nameIndex);
    String methodName = e1.getValue();

    if(!methodsList.contains(methodName))
      methodsList.addElement(methodName);

    if(methodEntry.getAccessFlags().isNative())
      if(!nativeMethodsList.contains(methodName))
        nativeMethodsList.addElement(methodName);
  }

  private void analyzeInnerClassesAttribute(InnerClassesAttribute attribute) {
    InnerClassEntry[] innerClasses = attribute.getInnerClasses();

    for(int i=1; i < innerClasses.length; i++) {
      InnerClassEntry innerClass = innerClasses[i];
      short innerNameIndex  = innerClass.getInnerNameIndex();

      if(innerNameIndex > 0) {
        Utf8Const utf2 = (Utf8Const)constPool.getEntry(innerNameIndex);
        String innerName = utf2.getValue();
        if(!innerNamesList.contains(innerName))
        innerNamesList.addElement(innerName);
      }
    }
  }

  public static void main(String[] argv) {
    String fromDir = "ppp";
    String classPath = System.getProperty("java.class.path") +
                       File.pathSeparator + fromDir;
    String mainClass = "p1/Test.class";
    NamesTable names  = new NamesTable(fromDir, classPath, mainClass,
                                       new Vector());

    CollectInfoTask task = new CollectInfoTask(names);

    try {
      task.solve("p1/Test.class");
    }
    catch(Exception e) {
      e.printStackTrace();
    }
    names.print();
  }

}
