// ObfuscateTask.java

package org.javalobby.task.obfuscator;

import java.io.*;
import java.util.*;

import org.javalobby.classfile.*;
import org.javalobby.classfile.*;
import org.javalobby.classfile.attribute.*;
import org.javalobby.classfile.instruction.*;
import org.javalobby.task.basic.*;


public class ObfuscateTask extends Task {
  protected final static int CLASS_NAME  = 0;
  protected final static int MEMBER_NAME = 1;
  protected final static int INNER_NAME  = 2;

  protected NamesTable names;

  private ConstPool constPool;
  private String qualifiedClassName;

  private boolean packageYes, classYes, fieldYes, methodYes, anonymousYes;

  public ObfuscateTask(String to, NamesTable names, boolean anonymousYes,
                       boolean[] conditions) {
    super(to);

    this.names = names;

    this.anonymousYes = anonymousYes;
    this.packageYes   = conditions[0];
    this.classYes     = conditions[1];
    this.fieldYes     = conditions[2];
    this.methodYes    = conditions[3];
  }

  public String getDestinationFileName() {
    String destination = super.getDestinationFileName();
    String newClassName = findNewValue(null, qualifiedClassName, CLASS_NAME);

    if(newClassName == null || newClassName.equals(qualifiedClassName))
      return destination;

    int pos = destination.indexOf("#");
    if(pos != -1)
      return destination.substring(0, pos) + "#" +
             newClassName.replace('/', File.separatorChar) + ".class";

    return destination;
  }

  public void body(ClassFile classFile) {
    qualifiedClassName = classFile.getClassName();

    constPool = classFile.getConstPool();
    EntryCollection fields     = classFile.fields();
    EntryCollection methods    = classFile.methods();
    EntryCollection attributes = classFile.attributes();

    if(fieldYes) {
      for(short i=0; i < fields.size(); i++)
        obfuscateFieldName((FieldEntry)fields.elementAt(i));

      for(short i=1; i < constPool.size(); i++) {
        PoolEntry e = (PoolEntry)constPool.elementAt(i);
        if(e == null) continue;

        if(e instanceof FieldConst)
          obfuscateFieldName((FieldConst)e);
      }
    }

    if(methodYes) {
      for(short i=0; i < methods.size(); i++)
        obfuscateMethodName((MethodEntry)methods.elementAt(i));

      for(short i=1; i < constPool.size(); i++) {
        PoolEntry e = (PoolEntry)constPool.elementAt(i);
        if(e == null) continue;

        if(e instanceof MethodConst) {
          obfuscateMethodName((MethodConst)e);
        }
      }
    }

    if(anonymousYes || packageYes || classYes) {
      for(short i=0; i < fields.size(); i++)
        obfuscateFieldSignature((FieldEntry)fields.elementAt(i));

      for(short i=0; i < methods.size(); i++) {
        MethodEntry e = (MethodEntry)methods.elementAt(i);
        obfuscateMethodSignature(e);
        obfuscateClassLiterals(e);
      }

      for(short i=1; i < constPool.size(); i++) {
        PoolEntry e = (PoolEntry)constPool.elementAt(i);
        if(e == null) continue;

        if(e instanceof ClassConst)
          obfuscateClassConst((ClassConst)e);
        else if(e instanceof FieldConst)
          obfuscateFieldSignature((FieldConst)e);
        else if(e instanceof MethodConst)
          obfuscateMethodSignature((MethodConst)e);
      }

      for(short i=0; i < attributes.size(); i++) {
        AttributeEntry attribute = (AttributeEntry)attributes.elementAt(i);
        String attrName = attribute.getName(constPool);
        if(attrName.equals(AttributeEntry.INNER_CLASSES)) {
          obfuscateInnerClassAttribute(new InnerClassesAttribute(attribute));
        }
      }
    }
  }

  private void obfuscateFieldName(FieldEntry e) {
    short nameIndex = e.getNameIndex();
    short valueIndex = e.getValueIndex();

    Utf8Const utf  = (Utf8Const)constPool.getEntry(nameIndex);
    String oldName = utf.getValue();

    obfuscateName(nameIndex, qualifiedClassName, oldName);
  }

  private void obfuscateFieldName(FieldConst e) {
    short nameIndex = e.getNameIndex();
    short valueIndex = e.getValueIndex();

    ClassConst e1       = (ClassConst)constPool.getEntry(nameIndex);
    NameAndTypeConst e2 = (NameAndTypeConst)constPool.getEntry(valueIndex);

    short nameIndex1  = e1.getNameIndex();
    short nameIndex2  = e2.getNameIndex();

    Utf8Const utf0   = (Utf8Const)constPool.getEntry(nameIndex1);
    Utf8Const utf1   = (Utf8Const)constPool.getEntry(nameIndex2);

    String className = utf0.getValue();
    String oldName   = utf1.getValue();

    obfuscateName(nameIndex2, className, oldName);
  }

  private void obfuscateMethodName(MethodEntry e) {
    short nameIndex  = e.getNameIndex();
    short valueIndex = e.getValueIndex();

    Utf8Const utf1 = (Utf8Const)constPool.getEntry(nameIndex);
    String oldName = utf1.getValue();

    obfuscateName(nameIndex, qualifiedClassName, oldName);
  }

  private void obfuscateClassLiterals(MethodEntry e) {
    try {
      findHiddenClasses(e);
    }
    catch(Exception ex) {
      ex.printStackTrace();
    }
  }

  private void findHiddenClasses(MethodEntry methodEntry) throws Exception {
    AttributeEntry attribute =
                   methodEntry.getAttribute(AttributeEntry.CODE, constPool);

    if(attribute != null) {
      CodeAttribute codeAttribute = new CodeAttribute(attribute);

      analyzeMethodBody(codeAttribute.getMethodBody().getInstructions());

      AttributeEntry[] attributes = codeAttribute.getAttributes();
      for(int i=0; i < attributes.length; i++) {
        String attrName = attributes[i].getName(constPool);
        if(attrName.equals(AttributeEntry.LOCAL_VARIABLE_TABLE)) {
          LocalVariableTableAttribute lvta =
                            new LocalVariableTableAttribute(attributes[i]);
          LocalVariableEntry[] localVariables = lvta.getLocalVariables();

          for(int j=0; j < localVariables.length; j++) {
            obfuscateUtf8Const(localVariables[j].getDescriptionIndex());
          }
        }
      }
    }
  }

  private void analyzeMethodBody(Instruction[] instructions) {
    for(int i=0; i < instructions.length; i++) {
      Instruction instruction = instructions[i];

      byte tag = instruction.getTag();
      OpcodeDescription description = OpcodeTable.getDescription(tag);

      if(description.length > 1) {
        RankTable rankTable = ParameterRanking.getInstance();
        String group = rankTable.getRank(tag).key;

        if(group.equals(ParameterRanking.METHOD_RANK) ||
           group.equals(ParameterRanking.IMETHOD_RANK)) {
          String str = instruction.resolve(constPool)[1];
          StringTokenizer st = new StringTokenizer(str);
          if(st.hasMoreTokens()) {
            st.nextToken();  // METHOD or IMETHOD
            st.nextToken(); //signature

            String methodName = st.nextToken();

            if(methodName.equals("forName") || methodName.startsWith("class$")) {
              StringConst e = findStringConst(instructions, i);
              if(e != null)
                obfuscateStringConst(e);
            }
          }
        }
      }
    }
  }

  private StringConst findStringConst(Instruction[] instructions, int j) {
    for(int cnt = j-1; cnt >= 0; cnt--) {
      Instruction instruction = instructions[cnt];

      byte tag = instruction.getTag();
      OpcodeDescription description = OpcodeTable.getDescription(tag);

      if(description.length > 1) {
        RankTable rankTable = ParameterRanking.getInstance();
        String group = rankTable.getRank(tag).key;

        if(group.equals(ParameterRanking.LDC_RANK)) {
          byte[] buffer = instruction.getParameter();

          short index = 0;
          if(buffer.length == 1) {
            index = (short)(buffer[0] & 0xff);
          }
          else if(buffer.length == 2) {
            index = (short)((buffer[0] & 0xff) << 8 | (buffer[1] & 0xff));
          }

          PoolEntry entry = constPool.getEntry(index);
          if(entry instanceof StringConst)
            return (StringConst)entry;
        }
      }
    }

    return null;
  }

  private void obfuscateMethodName(MethodConst e) {
    short nameIndex = e.getNameIndex();
    short valueIndex = e.getValueIndex();

    ClassConst e1       = (ClassConst)constPool.getEntry(nameIndex);
    NameAndTypeConst e2 = (NameAndTypeConst)constPool.getEntry(valueIndex);

    short nameIndex1  = e1.getNameIndex();
    short nameIndex2  = e2.getNameIndex();
    short valueIndex2 = e2.getValueIndex();

    Utf8Const utf0   = (Utf8Const)constPool.getEntry(nameIndex1);
    Utf8Const utf1   = (Utf8Const)constPool.getEntry(nameIndex2);

    String className = utf0.getValue();
    String oldName   = utf1.getValue();

    obfuscateName(nameIndex2, className, oldName);
  }

 private void obfuscateName(short nameIndex, String className,
                            String oldName) {
    String oldValue  = oldName;
    String newValue = findNewValue(className, oldValue, MEMBER_NAME);

    if(newValue != null && !oldValue.equals(newValue)) {
      String newName     = newValue;
      constPool.modifyUTF(nameIndex, newName);

      fireNotification(oldName + " -> " + newName, "renaming name");
    }
  }

  private void obfuscateClassConst(ClassConst e) {
    short index     = e.getNameIndex();
    Utf8Const utf   = (Utf8Const)constPool.getEntry(index);
    String oldValue = utf.getValue();
    String newValue = findNewValue(qualifiedClassName, oldValue, CLASS_NAME);

    if(newValue != null && !oldValue.equals(newValue)) {
      constPool.modifyUTF(index, newValue);
      fireNotification(oldValue + " -> " + newValue,
                       "renaming class const");
    }
  }

  private void obfuscateStringConst(StringConst e) {
    short index      = e.getNameIndex();
    Utf8Const utf    = (Utf8Const)constPool.getEntry(index);
    String oldValue  = utf.getValue();
    String oldValue2 = oldValue.replace('.', '/');
    String newValue  = findNewValue(qualifiedClassName,
                             oldValue2, CLASS_NAME);
    if(newValue != null) {
      String newValue2 = newValue.replace('/', '.');
      if(!oldValue2.equals(newValue)) {
        constPool.modifyUTF(index, newValue2);
        fireNotification(oldValue + " -> " + newValue2,
                         "renaming string const");
      }
    }
  }

  private void obfuscateUtf8Const(short index) {
    PoolEntry entry = constPool.getEntry(index);

    if(entry instanceof Utf8Const) {
      Utf8Const utf8 = (Utf8Const)entry;

      String oldValue = utf8.getValue();
      String newValue = change(oldValue);

      if(newValue != null) {
        constPool.modifyUTF(index, newValue);
        fireNotification(oldValue + " -> " + newValue,
                         "renaming utf const in local variable table");
      }
    }
  }

  private void obfuscateFieldSignature(FieldEntry e) {
    short valueIndex = e.getValueIndex();

    obfuscateFieldSignature(valueIndex);
  }

  private void obfuscateFieldSignature(FieldConst e) {
    short valueIndex    = e.getValueIndex();
    NameAndTypeConst e2 = (NameAndTypeConst)constPool.getEntry(valueIndex);
    short valueIndex2   = e2.getValueIndex();

    obfuscateFieldSignature(valueIndex2);
  }

  private void obfuscateFieldSignature(short index) {
    Utf8Const utf  = (Utf8Const)constPool.getEntry(index);
    String oldType = utf.getValue();

    String newType = change(oldType);

    if(newType != null) {
      constPool.modifyUTF(index, newType);
      fireNotification(oldType + " -> " + newType,
                       "renaming field signature");
    }

    /*StringBuffer prefix = new StringBuffer();
    int cnt = 0;
    while(oldType.charAt(cnt) == '[') {
      cnt++;
      prefix.append("[");
    }

    if(oldType.charAt(cnt) == 'L') {
      String oldTypeRefine = oldType.substring(cnt+1, oldType.length()-1);
      String newTypeRefine = findNewValue(null, oldTypeRefine, CLASS_NAME);
      if(newTypeRefine != null && !oldTypeRefine.equals(newTypeRefine)) {
        String newType = prefix + "L" + newTypeRefine + ";";
        constPool.modifyUTF(index, newType);
        fireNotification(oldType + " -> " + newType,
                         "renaming field signature");
      }
    }
    */
  }


  private void obfuscateMethodSignature(MethodEntry e) {
    short valueIndex = e.getValueIndex();

    obfuscateMethodSignature(valueIndex, (short)0);
  }

  private void obfuscateMethodSignature(MethodConst e) {
    short valueIndex    = e.getValueIndex();
    NameAndTypeConst e2 = (NameAndTypeConst)constPool.getEntry(valueIndex);
    short valueIndex2   = e2.getValueIndex();

    short nameIndex2   = e2.getNameIndex();

    obfuscateMethodSignature(valueIndex2, nameIndex2);
  }

  private void obfuscateMethodSignature(short index, short nameIndex2) {
    Utf8Const utf       = (Utf8Const)constPool.getEntry(index);

    String oldSignature = utf.getValue();
    MethodSignature methodSignature = new MethodSignature(oldSignature);
    Signature[] parameters = methodSignature.getParameters();

    boolean changes = false;
    for(int i=0; i < parameters.length; i++) {
      String oldValue = parameters[i].getKey();
      String newValue = change(oldValue);

      if(newValue != null) {
        changes = true;
        parameters[i] = new Signature(newValue);
      }
    }

    Signature returnType = methodSignature.getReturnType();

    String oldReturnType = returnType.getKey();
    String newReturnType = change(oldReturnType);

//    System.out.println("oldReturnType " + oldReturnType);
//    System.out.println("newReturnType " + newReturnType);

    if(newReturnType != null) {
      changes = true;
      returnType.setKey(newReturnType);
    }

    if(changes) {
      String newSignature = methodSignature.toString();
      constPool.modifyUTF(index, newSignature);
      fireNotification(oldSignature + " -> " + newSignature,
                       "renaming method signature");
    }
  }

  private String change(String oldValue) {
    StringBuffer prefix = new StringBuffer();
    int cnt = 0;
    while(oldValue.charAt(cnt) == '[') {
      cnt++;
      prefix.append("[");
    }

    if(oldValue.charAt(cnt) == 'L') {
      String oldValueRefine = oldValue.substring(cnt+1, oldValue.length()-1);
      String newValueRefine = findNewValue(null, oldValueRefine, CLASS_NAME);

      if(newValueRefine != null && !oldValueRefine.equals(newValueRefine)) {
        String newValue = prefix + "L" + newValueRefine + ";";

        return newValue;
      }
    }

    return null;
  }

  private void obfuscateInnerClassAttribute(InnerClassesAttribute attribute) {
    InnerClassEntry[] innerClasses = attribute.getInnerClasses();

    for(int i=1; i < innerClasses.length; i++) {
      InnerClassEntry innerClass = innerClasses[i];
      short innerClassIndex = innerClass.getInnerClassIndex();
      short innerNameIndex  = innerClass.getInnerNameIndex();

      if(innerNameIndex > 0) {
        ClassConst classConst = (ClassConst)constPool.getEntry(innerClassIndex);
        short nameIndex = classConst.getNameIndex();
        Utf8Const utf1   = (Utf8Const)constPool.getEntry(nameIndex);
        String newName = utf1.getValue();
        int pos = newName.indexOf('$');
        if(pos != -1) {
          Utf8Const utf2 = (Utf8Const)constPool.getEntry(innerNameIndex);
          String oldValue = utf2.getValue();
          String newValue = findNewValue(qualifiedClassName, oldValue, INNER_NAME);
          if(newValue != null) {
            if(!oldValue.equals(newValue)) {
              constPool.modifyUTF(innerNameIndex, newValue);
              fireNotification(oldValue + " -> " + newValue,
                               "renaming inner classes attribute");
            }
          }
        }
      }
    }
  }

  protected String findNewValue(String className, String value, int type) {
    if(type == CLASS_NAME)
      return names.getObfuscatedClassName(value);
    else if(type == MEMBER_NAME)
      return names.getObfuscatedName(className, value);
    else if(type == INNER_NAME)
      return names.getObfuscatedInnerName(className, value);

    return null;
  }


  public static void main(String[] argv) {
    String fromDir   = ".";
    String mainClass = "p1/Test.class";
    String classPath = System.getProperty("java.class.path") +
                       File.pathSeparator + fromDir;

    NamesTable names  = new NamesTable(fromDir, classPath, mainClass,
                                       new Vector());

    boolean anonymousPackageMode = false;
    boolean[] conditions = new boolean[] { true, true, true, false };

    CollectInfoTask collectInfoTask = new CollectInfoTask(names);
    ObfuscateTask obfuscateTask =
         new ObfuscateTask("Result", names, anonymousPackageMode, conditions);

    try {
      collectInfoTask.solve(mainClass);

      names.generateAllNames(anonymousPackageMode, conditions);

      names.print();

      obfuscateTask.solve("p1/Test.class");
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

}
