// BytecodeClassAnalyzer.java

package org.javalobby.task.util;

import java.lang.reflect.*;
import java.io.*;
import java.util.*;
import java.util.zip.*;

import org.javalobby.classfile.*;
import org.javalobby.classfile.*;
import org.javalobby.task.util.*;

public class BytecodeClassAnalyzer implements ClassAnalyzer {

  private String currentClassName;
  private OneToManyTable classToParentsTable;
  private OneToManyTable classToNamesTable;

  private CustomClassPath classPathVariable;

  public BytecodeClassAnalyzer(String classPath,
                               OneToManyTable classToParentsTable) {
    this(classPath, classToParentsTable, null);
  }

  public BytecodeClassAnalyzer(String classPath,
                               OneToManyTable classToParentsTable,
                               OneToManyTable classToNamesTable) {
    classPathVariable = new CustomClassPath(classPath);

    this.classToParentsTable = classToParentsTable;
    this.classToNamesTable   = classToNamesTable;
  }

  public void analyze(String className) {
    currentClassName = className.replace('.', '/');

    seek(currentClassName);
  }

  public OneToManyTable getParents() {
    return classToParentsTable;
  }

  public OneToManyTable getNames() {
    return classToNamesTable;
  }

  private void seek(String className) {
    try {
      Class clazz = Class.forName(className.replace('/', '.'));
      getSuperclass(clazz);
      getInterfaces(clazz);
      if(classToNamesTable != null)
        getNames(clazz);
    }
    catch(Exception e) {
      seek0(className);
    }
  }

  private void getSuperclass(Class clazz) {
    Class parentClass = clazz.getSuperclass();
    if(parentClass != null) {
      String parentClassName = parentClass.getName().replace('.', '/');
      classToParentsTable.put(currentClassName, parentClassName);
      getSuperclass(parentClass);
    }
  }

  private void getInterfaces(Class clazz) {
    Class[] interfaces = clazz.getInterfaces();

    for(int i=0; i < interfaces.length; i++) {
      String interfaceName = interfaces[i].getName().replace('.', '/');
      classToParentsTable.put(currentClassName, interfaceName);
      getInterfaces(interfaces[i]);
    }
  }

  private void getNames(Class clazz) {
    String className = clazz.getName().replace('.', '/');

    Method methods[] = clazz.getDeclaredMethods();

    for(int i=0; i < methods.length; i++)
      classToNamesTable.put(className, methods[i].getName());

    Field fields[] = clazz.getDeclaredFields();

    for(int i=0; i < fields.length; i++)
      classToNamesTable.put(className, fields[i].getName());
  }

  private void seek0(String className) {
    ClassFile classFile = null;

    Enumeration e = classPathVariable.elements();
    while(e.hasMoreElements() && classFile == null) {
      String entry = (String)e.nextElement();
      try {
        classFile = read(entry, className);
      }
      catch(Exception ex) {
        continue;
      }
    }

    if(classFile != null)
      seek0(classFile);
    else
      System.err.println("Error reading class " + className);
  }

  public void seek0(ClassFile classFile) {
    getSuperclasses0(classFile);
    getInterfaces0(classFile);
    if(classToNamesTable != null)
      getNames0(classFile);
  }

  private ClassFile read(String entry, String className) throws Exception {

    ClassFile classFile = new ClassFile();

    boolean isArchive = !(new File(entry)).isDirectory();

    if(isArchive) {
      ZipFile zipFile   = new ZipFile(entry);
      String entryName  = className + ".class";
      ZipEntry zipEntry = zipFile.getEntry(entryName);
      if(zipEntry == null) return null;

      classFile.read(zipFile.getInputStream(zipEntry));
    }
    else {
      String fileName = className.replace('/', File.separatorChar) + ".class";
      try {
        classFile.read(entry + File.separator + fileName);
      }
      catch(Exception ex) {
        String onlyName = fileName;
        int pos = fileName.lastIndexOf(File.separatorChar);
        if(pos != -1)
          onlyName = fileName.substring(pos);

        classFile.read(entry + File.separator + onlyName);
      }
    }

    return classFile;
  }

  private void getSuperclasses0(ClassFile classFile) {
    String parentClassName = classFile.getSuperclassName();

    if(parentClassName.equals("java/lang/Object")) {
      AccessFlags accessFlags = classFile.getAccessFlags();
      if(!accessFlags.isInterface()) {
        classToParentsTable.put(currentClassName, parentClassName);
      }
    }
    else {
      classToParentsTable.put(currentClassName, parentClassName);
      seek(parentClassName);
    }
  }

  private void getInterfaces0(ClassFile classFile) {
    String[] interfaces = classFile.getInterfacesNames();

    for(int i=0; i < interfaces.length; i++) {
      String interfaceName = interfaces[i];

      if(!interfaceName.equals("java/lang/Object")) {
        classToParentsTable.put(currentClassName, interfaceName);
        seek(interfaceName);
      }
    }
  }

  private void getNames0(ClassFile classFile) {
    String className        = classFile.getClassName();
    ConstPool constPool     = classFile.getConstPool();

    EntryCollection methods = classFile.methods();

    for(short i=0; i < methods.size(); i++) {
      MethodEntry e    = (MethodEntry)methods.elementAt(i);
      short nameIndex  = e.getNameIndex();
      short valueIndex = e.getValueIndex();

      Utf8Const e1 = (Utf8Const)constPool.getEntry(nameIndex);
      classToNamesTable.put(className, e1.getValue());
    }

    EntryCollection fields = classFile.fields();

    for(short i=0; i < fields.size(); i++) {
      FieldEntry e    = (FieldEntry)fields.elementAt(i);
      short nameIndex  = e.getNameIndex();
      short valueIndex = e.getValueIndex();

      Utf8Const e1 = (Utf8Const)constPool.getEntry(nameIndex);
      classToNamesTable.put(className, e1.getValue());
    }

  }


  public static void main(String[] args) {

    String classPath = "d:\\jdk1.1\\lib\\classes.zip;d:\\swing\\swingall.jar;.;classes;..\\ppp";

    OneToManyTable classToParentsTable = new OneToManyTable();
    OneToManyTable classToNamesTable = new OneToManyTable();

    ClassAnalyzer analyzer = new BytecodeClassAnalyzer(classPath,
                                 classToParentsTable, classToNamesTable);

    analyzer.analyze("classfile/IMethodConst");
    analyzer.analyze("java/lang/String");

    System.out.println("Class->Parents:");
    classToParentsTable.print();

    System.out.println("Class->Names:");
    classToNamesTable.print();
  }

}

